# coding: utf-8

from __future__ import unicode_literals
import json
from mock import mock_open, patch
import pytest
from six import BytesIO
from boxsdk.config import API
from boxsdk.exception import BoxAPIException
from boxsdk.object.file import File


# pylint:disable=protected-access
# pylint:disable=redefined-outer-name

@pytest.fixture()
def mock_accelerator_upload_url_for_update():
    return 'https://upload.box.com/api/2.0/files/fake_file_id/content?upload_session_id=123'


@pytest.fixture(scope='function')
def mock_accelerator_response_for_update(make_mock_box_request, mock_accelerator_upload_url_for_update):
    mock_response, _ = make_mock_box_request(
        response={
            'upload_url': mock_accelerator_upload_url_for_update,
            'download_url': None,
        }
    )
    return mock_response


def test_delete_file(test_file, mock_box_session, etag, if_match_header):
    test_file.delete(etag=etag)
    expected_url = test_file.get_url()
    mock_box_session.delete.assert_called_once_with(
        expected_url,
        expect_json_response=False,
        params={},
        headers=if_match_header,
    )


def test_download_to(test_file, mock_box_session, mock_content_response):
    expected_url = test_file.get_url('content')
    mock_box_session.get.return_value = mock_content_response
    mock_writeable_stream = BytesIO()
    test_file.download_to(mock_writeable_stream)
    mock_writeable_stream.seek(0)
    assert mock_writeable_stream.read() == mock_content_response.content
    mock_box_session.get.assert_called_once_with(expected_url, expect_json_response=False, stream=True)


def test_get_content(test_file, mock_box_session, mock_content_response):
    expected_url = test_file.get_url('content')
    mock_box_session.get.return_value = mock_content_response
    file_content = test_file.content()
    assert file_content == mock_content_response.content
    mock_box_session.get.assert_called_once_with(expected_url, expect_json_response=False)


@pytest.mark.parametrize('is_stream', (True, False))
def test_update_contents(
        test_file,
        mock_box_session,
        mock_content_response,
        mock_upload_response,
        mock_file_path,
        etag,
        upload_using_accelerator,
        mock_accelerator_response_for_update,
        mock_accelerator_upload_url_for_update,
        upload_using_accelerator_fails,
        if_match_header,
        is_stream,
):
    expected_url = test_file.get_url('content').replace(API.BASE_API_URL, API.UPLOAD_URL)
    if upload_using_accelerator:
        if upload_using_accelerator_fails:
            mock_box_session.options.side_effect = BoxAPIException(400)
        else:
            mock_box_session.options.return_value = mock_accelerator_response_for_update
            expected_url = mock_accelerator_upload_url_for_update

    mock_box_session.post.return_value = mock_upload_response

    if is_stream:
        mock_file_stream = BytesIO(mock_content_response.content)
        new_file = test_file.update_contents_with_stream(
            mock_file_stream,
            etag=etag,
            upload_using_accelerator=upload_using_accelerator,
        )
    else:
        mock_file = mock_open(read_data=mock_content_response.content)
        mock_file_stream = mock_file.return_value
        with patch('boxsdk.object.file.open', mock_file, create=True):
            new_file = test_file.update_contents(
                mock_file_path,
                etag=etag,
                upload_using_accelerator=upload_using_accelerator,
            )

    mock_files = {'file': ('unused', mock_file_stream)}
    mock_box_session.post.assert_called_once_with(
        expected_url,
        expect_json_response=False,
        files=mock_files,
        headers=if_match_header,
    )
    assert isinstance(new_file, File)
    assert new_file.object_id == test_file.object_id
    assert 'id' in new_file
    assert new_file['id'] == test_file.object_id
    assert not hasattr(new_file, 'entries')
    assert 'entries' not in new_file


def test_update_contents_with_stream_does_preflight_check_if_specified(
        test_file,
        preflight_check,
        file_size,
        preflight_fails,
        mock_box_session,
):
    with patch.object(File, 'preflight_check', return_value=None):
        kwargs = {'file_stream': BytesIO(b'some bytes')}
        if preflight_check:
            kwargs['preflight_check'] = preflight_check
            kwargs['preflight_expected_size'] = file_size
        if preflight_fails:
            test_file.preflight_check.side_effect = BoxAPIException(400)
            with pytest.raises(BoxAPIException):
                test_file.update_contents_with_stream(**kwargs)
        else:
            test_file.update_contents_with_stream(**kwargs)

        if preflight_check:
            assert test_file.preflight_check.called_once_with(size=file_size)
            if preflight_fails:
                assert not mock_box_session.post.called
            else:
                assert mock_box_session.post.called
        else:
            assert not test_file.preflight_check.called


@patch('boxsdk.object.file.open', mock_open(read_data=b'some bytes'), create=True)
def test_update_contents_does_preflight_check_if_specified(
        test_file,
        mock_file_path,
        preflight_check,
        file_size,
        preflight_fails,
        mock_box_session,
):
    with patch.object(File, 'preflight_check', return_value=None):
        kwargs = {'file_path': mock_file_path}
        if preflight_check:
            kwargs['preflight_check'] = preflight_check
            kwargs['preflight_expected_size'] = file_size
        if preflight_fails:
            test_file.preflight_check.side_effect = BoxAPIException(400)
            with pytest.raises(BoxAPIException):
                test_file.update_contents(**kwargs)
        else:
            test_file.update_contents(**kwargs)

        if preflight_check:
            assert test_file.preflight_check.called_once_with(size=file_size)
            if preflight_fails:
                assert not mock_box_session.post.called
            else:
                assert mock_box_session.post.called
        else:
            assert not test_file.preflight_check.called


@pytest.mark.parametrize('prevent_download', (True, False))
def test_lock(test_file, mock_box_session, mock_file_response, prevent_download):
    expected_url = test_file.get_url()
    mock_box_session.put.return_value = mock_file_response
    test_file.lock(prevent_download)
    mock_box_session.put.assert_called_once_with(
        expected_url,
        data=json.dumps({'lock': {'is_download_prevented': prevent_download, 'type': 'lock'}}),
        params=None,
        headers=None,
    )


def test_unlock(test_file, mock_box_session, mock_file_response):
    expected_url = test_file.get_url()
    mock_box_session.put.return_value = mock_file_response
    test_file.unlock()
    mock_box_session.put.assert_called_once_with(
        expected_url,
        data=json.dumps({'lock': None}),
        params=None,
        headers=None,
    )


@pytest.mark.parametrize(
    'size, name, expected_data',
    [
        # Test case for specifying the name of the file for preflight
        (
            100,
            'foo.txt',
            json.dumps({'size': 100, 'name': 'foo.txt'}),
        ),

        # Test case for omitting the name of the file for preflight
        (
            200,
            None,
            json.dumps({'size': 200})
        ),
    ]
)
def test_preflight_check(
        test_file,
        mock_object_id,
        mock_box_session,
        size,
        name,
        expected_data,
):
    kwargs = {'size': size}
    if name:
        kwargs['name'] = name
    test_file.preflight_check(**kwargs)
    mock_box_session.options.assert_called_once_with(
        url='{0}/files/{1}/content'.format(
            API.BASE_API_URL,
            mock_object_id,
        ),
        expect_json_response=False,
        data=expected_data,
    )


def test_get_shared_link_download_url(
        test_file,
        mock_box_session,
        shared_link_access,
        shared_link_unshared_at,
        shared_link_password,
        shared_link_can_preview,
        test_url,
        etag,
        if_match_header,
):
    # pylint:disable=redefined-outer-name, protected-access
    expected_url = test_file.get_url()
    mock_box_session.put.return_value.json.return_value = {'shared_link': {'url': None, 'download_url': test_url}}
    expected_data = {'shared_link': {}}
    if shared_link_access is not None:
        expected_data['shared_link']['access'] = shared_link_access
    if shared_link_unshared_at is not None:
        expected_data['shared_link']['unshared_at'] = shared_link_unshared_at.isoformat()
    if shared_link_can_preview is not None:
        expected_data['shared_link']['permissions'] = permissions = {}
        permissions['can_preview'] = shared_link_can_preview
    if shared_link_password is not None:
        expected_data['shared_link']['password'] = shared_link_password
    url = test_file.get_shared_link_download_url(
        etag=etag,
        access=shared_link_access,
        unshared_at=shared_link_unshared_at,
        password=shared_link_password,
        allow_preview=shared_link_can_preview,
    )
    mock_box_session.put.assert_called_once_with(
        expected_url,
        data=json.dumps(expected_data),
        headers=if_match_header,
        params=None,
    )
    assert url == test_url



# coding: utf-8

from __future__ import unicode_literals

from six.moves import map   # pylint:disable=redefined-builtin


__all__ = list(map(str, ['collaboration', 'events', 'event', 'file', 'folder', 'group', 'group_membership', 'recent_item', 'search', 'user']))



# coding: utf-8

from __future__ import division, unicode_literals

from datetime import datetime, timedelta
from threading import Event

from bottle import request
from sqlalchemy import event

from test.functional.mock_box.db_model.event_model import EventModel
from test.functional.mock_box.db_model.file_model import FileModel
from test.functional.mock_box.db_model.folder_model import FolderModel
from test.functional.mock_box.util import json_utils as json


class EventBehavior(object):
    def __init__(self, db_session):
        self._db_session = db_session
        self._subscribe_event = Event()
        self._epoch = datetime.utcnow()

        def handle_event(*_):
            self._subscribe_event.set()

        event.listen(EventModel, 'after_insert', handle_event)

    def _ms_since_epoch(self, when):
        delta = when - self._epoch
        total_seconds = (delta.microseconds + (delta.seconds + delta.days * 24 * 3600) * 10 ** 6) / 10 ** 6
        return int(total_seconds * 1000)

    def _ms_after_epoch(self, when):
        return self._epoch + timedelta(seconds=when / 1000)

    def get_events(self):
        stream_position = request.params.get('stream_position', '0')
        limit = int(request.params.get('limit', '100'))
        if stream_position == 'now':
            return json.dumps({
                'total_count': 0,
                'next_stream_position': self._ms_since_epoch(datetime.utcnow()),
                'entries': [],
            })
        stream_position = int(stream_position)
        event_query = self._db_session.query(EventModel).filter(
            EventModel.stream_position >= self._ms_after_epoch(stream_position or 1),
        )
        total_count = event_query.count()
        results = event_query.order_by(EventModel.sequence_id).limit(limit).all()
        max_stream_position = max(e.stream_position for e in results) if results else self._epoch
        next_stream_position = datetime.utcnow() if total_count < limit else max_stream_position - timedelta(seconds=5)
        file_ids = [e.source_id for e in results if e.source_type == 'file']
        folder_ids = [e.source_id for e in results if e.source_type == 'folder']
        file_records = self._db_session.query(FileModel).filter(FileModel.file_id.in_(file_ids)).all() if file_ids else []
        folder_records = self._db_session.query(FolderModel).filter(FolderModel.folder_id.in_(folder_ids)).all() if folder_ids else []
        if results:
            results = [{
                'type': 'event',
                'event_id': e.event_id,
                'event_type': e.event_type,
                'source_id': e.source_id,
                'source': next(
                    (f for f in file_records if f.file_id == e.source_id),
                    None
                ) or next((f for f in folder_records if f.folder_id == e.source_id), None),
            } for e in results]
        return json.dumps({
            'chunk_size': total_count,
            'next_stream_position': self._ms_since_epoch(next_stream_position),
            'entries': results,
        })

    @staticmethod
    def get_long_poll_url():
        return {
            'chunk_size': 1,
            'entries': [{
                'type': 'realtime_server',
                'url': 'http://localhost:8089/subscribe?channel=cc807c9c4869ffb1c81a&stream_type=all',
                'ttl': 10,
                'max_retries': 10,
                'retry_timeout': 610,
            }]
        }

    def subscribe(self):
        if self._subscribe_event.isSet():
            self._subscribe_event.clear()
            return {
                'message': 'new_change'
            }
        got_event = self._subscribe_event.wait(timeout=600)
        if got_event:
            self._subscribe_event.clear()
            return {
                'message': 'new_change'
            }
        else:
            return {
                'message': 'reconnect'
            }



# coding: utf-8

from __future__ import unicode_literals
import pytest
from boxsdk.util.shared_link import get_shared_link_header


@pytest.fixture(params=('mock_shared_link', 'https://app.box.com/s/q2i1024dvguiads6mzj2avsq9hmz43du'))
def shared_link(request):
    return request.param


@pytest.fixture(params=(None, 'shared_link_password'))
def password(request):
    return request.param


def test_get_shared_link_header(shared_link, password):
    # pylint:disable=redefined-outer-name
    header = get_shared_link_header(shared_link, password)
    assert 'BoxApi' in header
    assert shared_link in header['BoxApi']
    if password is not None:
        assert password in header['BoxApi']



# coding: utf-8

from __future__ import unicode_literals


def get_shared_link_header(shared_link, password=None):
    """
    Gets the HTTP header required to use a shared link to grant access to a shared item.

    :param shared_link:
        The shared link.
    :type shared_link:
        `unicode`
    :param password:
        The password for the shared link.
    :type password:
        `unicode`
    :return:
        The item referred to by the shared link.
    :rtype:
        :class:`Item`
    """
    shared_link_password = '&shared_link_password={0}'.format(password) if password is not None else ''
    box_api_header = 'shared_link={0}{1}'.format(shared_link, shared_link_password)
    return {'BoxApi': box_api_header}



# coding: utf-8

from __future__ import unicode_literals, absolute_import
from functools import partial
import json

from .base_object import BaseObject
from ..config import API
from ..util.api_call_decorator import api_call


class Group(BaseObject):
    """Represents a Box group."""

    _item_type = 'group'

    @api_call
    def membership(self, starting_index=0, limit=100, include_page_info=False):
        """
        A generator over all the members of this Group. The paging in the API is transparently implemented
        inside the generator. By adjusting the page_size, the caller can control the chattiness of the API. Caller
        can also implement their owning paging and/or control exactly when an API is called by
        using the 'include_page_info' param as follows:

            for group, page_size, index in group.membership(..., include_page_info=True):
                # when index + 1 == page_size, the next iteration of this loop will
                # trigger an API call, unless we've reached the end of *all* the data.
                pass

        :param starting_index:
            The index at which to begin.
        :type starting_index:
            `int`
        :param limit:
            The maximum number of items to return in a page.
        :type limit:
            `int`
        :returns:
            A generator of GroupMembership instances. Or, if include_page_info
            is True, it is a generator of 3-tuples, where each tuple is
                1) GroupMembership instance
                2) Number of GroupMemberships returned by the last paged API call
                3) Index of *this* GroupMembership instance in the current page.
        :rtype:
            `generator` of :class:`GroupMembership` or, if include_page_info
            is True then `tuple` of (:class:`GroupMembership`, `int`, `int`)
        """
        url = self.get_url('memberships')

        membership_factory = partial(self.translator.translate("group_membership"), group=self)
        for group_membership_tuple in self._paging_wrapper(url, starting_index, limit, membership_factory):
            if include_page_info:
                yield group_membership_tuple
            else:
                group_membership, _, _ = group_membership_tuple
                yield group_membership

    @api_call
    def add_member(self, user, role):
        """
        Add the given user to this group under the given role

        :param user:
            The User to add to the group.
        :type user:
            :class:`User`
        :param role:
            The role for the user. TODO: determine valid options and create an Enum.
        :type role:
            `unicode`
        :returns:
            The new GroupMembership instance.
        :rtype:
            :class:`GroupMembership`
        """
        url = '{0}/group_memberships'.format(API.BASE_API_URL)
        body_attributes = {
            'user': {'id': user.object_id},
            'group': {'id': self.object_id},
            'role': role,
        }
        box_response = self._session.post(url, data=json.dumps(body_attributes))
        response = box_response.json()

        return self.translator.translate(response['type'])(self._session, response['id'], response, user=user, group=self)



# coding: utf-8

from __future__ import unicode_literals, absolute_import
import json
from mock import Mock, PropertyMock
import pytest

from boxsdk.network.default_network import DefaultNetworkResponse
from boxsdk.pagination.limit_offset_based_object_collection import LimitOffsetBasedObjectCollection
from boxsdk.session.box_session import BoxResponse, BoxSession
from .box_object_collection_test_base import BoxObjectCollectionTestBase


class TestLimitOffsetBasedObjectCollection(BoxObjectCollectionTestBase):
    DEFAULT_LIMIT = 100

    @pytest.fixture()
    def mock_items_response(self, entries):
        """Baseclass override."""
        # pylint:disable=redefined-outer-name
        def get_response(limit, offset):
            mock_box_response = Mock(BoxResponse)
            mock_network_response = Mock(DefaultNetworkResponse)
            mock_box_response.network_response = mock_network_response
            mock_box_response.json.return_value = mock_json = {
                'entries': entries[offset:limit + offset],
                'total_count': len(entries),
                'limit': limit,
            }
            mock_box_response.content = json.dumps(mock_json).encode()
            mock_box_response.status_code = 200
            mock_box_response.ok = True
            return mock_box_response
        return get_response

    @pytest.fixture()
    def mock_session(self, translator, mock_items_response):
        """Baseclass override."""
        mock_box_session = Mock(BoxSession)
        type(mock_box_session).translator = PropertyMock(
            return_value=translator
        )

        def mock_items_side_effect(_, params):
            limit = min(params.get('limit', self.DEFAULT_LIMIT), self.DEFAULT_LIMIT)
            offset = params.get('offset', 0)
            return mock_items_response(limit, offset)

        mock_box_session.get.side_effect = mock_items_side_effect
        return mock_box_session

    def _object_collection_instance(self, session, limit=None, return_full_pages=False, starting_pointer=None):
        """Baseclass override."""
        if starting_pointer is None:
            starting_pointer = 0
        return LimitOffsetBasedObjectCollection(
            session,
            '/some/endpoint',
            limit=limit,
            return_full_pages=return_full_pages,
            offset=starting_pointer,
        )

    @pytest.mark.parametrize('return_full_pages', (True, False))
    def test_object_collection_sets_next_pointer_correctly(self, mock_session, return_full_pages):
        page_size = 10
        object_collection = LimitOffsetBasedObjectCollection(
            mock_session,
            '/some/endpoint',
            limit=page_size,
            return_full_pages=return_full_pages,
        )

        assert object_collection.next_pointer() == 0
        object_collection.next()
        assert object_collection.next_pointer() == page_size

        # Iterate to the last page, which doesn't return a full page.
        [_ for _ in object_collection]  # pylint:disable=pointless-statement
        assert object_collection.next_pointer() == self.NUM_ENTRIES

    def test_object_collection_raises_stop_iteration_when_starting_offset_is_too_far(self, mock_session, entries):
        """
        If the specified initial offset for the object collection is higher than the total number of items,
        then the first call to next() on the object collection should raise a StopIteration.
        """
        starting_offset = len(entries) + 10
        object_collection = self._object_collection_instance(
            mock_session,
            limit=5,
            return_full_pages=False,
            starting_pointer=starting_offset
        )
        with pytest.raises(StopIteration):
            object_collection.next()

    def test_object_collection_sets_limit_to_returned_value_if_originally_none(self, mock_session):
        # No limit specified
        object_collection = self._object_collection_instance(mock_session)
        object_collection.next()
        assert object_collection._limit == self.DEFAULT_LIMIT   # pylint:disable=protected-access

    def test_object_collection_sets_limit_to_returned_value_if_originally_too_high(self, mock_session):
        object_collection = self._object_collection_instance(mock_session, limit=(1 + self.DEFAULT_LIMIT))
        object_collection.next()
        assert object_collection._limit == self.DEFAULT_LIMIT   # pylint:disable=protected-access



# coding: utf-8

from __future__ import absolute_import, unicode_literals

from functools import wraps


__all__ = list(map(str, ['ChainMap']))


# pylint:disable=unused-import
try:
    from collections import ChainMap
except ImportError:
    from chainmap import ChainMap as _ChainMap

    # Make sure that `ChainMap` is a new-style class.
    @wraps(_ChainMap, updated=())
    class ChainMap(_ChainMap, object):
        __slots__ = ()



# coding: utf-8

from __future__ import unicode_literals
import json
from os.path import basename
from mock import mock_open, patch, Mock, MagicMock
import pytest
from six import BytesIO
from six.moves import zip  # pylint:disable=redefined-builtin,import-error
from boxsdk.config import API
from boxsdk.exception import BoxAPIException
from boxsdk.network.default_network import DefaultNetworkResponse
from boxsdk.object.file import File
from boxsdk.object.collaboration import Collaboration, CollaborationRole
from boxsdk.object.folder import Folder, FolderSyncState
from boxsdk.pagination.limit_offset_based_object_collection import LimitOffsetBasedObjectCollection
from boxsdk.pagination.marker_based_object_collection import MarkerBasedObjectCollection
from boxsdk.session.box_session import BoxResponse


# pylint:disable=protected-access
# pylint:disable=redefined-outer-name


@pytest.fixture()
def mock_new_upload_accelerator_url():
    return 'https://upload.box.com/api/2.0/files/content?upload_session_id=123'


@pytest.fixture(scope='function')
def mock_accelerator_response_for_new_uploads(make_mock_box_request, mock_new_upload_accelerator_url):
    mock_response, _ = make_mock_box_request(
        response={
            'upload_url': mock_new_upload_accelerator_url,
            'upload_token': None,
        }
    )
    return mock_response


@pytest.fixture()
def mock_items(mock_box_session, mock_object_id):
    return [
        {'type': 'file', 'id': mock_object_id},
        {'type': 'folder', 'id': mock_object_id},
        {'type': 'file', 'id': mock_object_id},
    ], [
        File(mock_box_session, mock_object_id),
        Folder(mock_box_session, mock_object_id),
        File(mock_box_session, mock_object_id),
    ]


@pytest.fixture()
def mock_items_response(mock_items):
    # pylint:disable=redefined-outer-name
    def get_response(limit, offset):
        items_json, items = mock_items
        mock_box_response = Mock(BoxResponse)
        mock_network_response = Mock(DefaultNetworkResponse)
        mock_box_response.network_response = mock_network_response
        mock_box_response.json.return_value = mock_json = {'entries': items_json[offset:limit + offset]}
        mock_box_response.content = json.dumps(mock_json).encode()
        mock_box_response.status_code = 200
        mock_box_response.ok = True
        return mock_box_response, items[offset:limit + offset]
    return get_response


def _assert_collaborator_added(test_folder, collaborator, mock_box_session, mock_collab_response, notify, role, can_view_path, data):
    mock_box_session.post.return_value = mock_collab_response
    collaboration = test_folder.add_collaborator(collaborator, role, notify, can_view_path)
    assert isinstance(collaboration, Collaboration)
    expected_url = API.BASE_API_URL + '/collaborations'
    params = {'notify': notify}
    mock_box_session.post.assert_called_once_with(expected_url, expect_json_response=True, data=data, params=params)


@pytest.mark.parametrize('accessible_by', ['user', 'group', 'email'])
@pytest.mark.parametrize('notify', [True, False])
@pytest.mark.parametrize('role', iter(CollaborationRole))
@pytest.mark.parametrize('can_view_path', [True, False])
def test_add_collaborator(test_folder, mock_user, mock_group, mock_box_session, mock_collab_response, accessible_by, notify, role, can_view_path):
    accessible_dict = {
        'user': (mock_user, {'id': mock_user.object_id, 'type': 'user'}),
        'group': (mock_group, {'id': mock_group.object_id, 'type': 'group'}),
        'email': ('foo@example.com', {'login': 'foo@example.com', 'type': 'user'}),
    }

    invitee, mock_accessible_by = accessible_dict[accessible_by]

    body_params = {
        'item': {'id': test_folder.object_id, 'type': 'folder'},
        'accessible_by': mock_accessible_by,
        'role': role,
    }
    if can_view_path:
        body_params['can_view_path'] = True
    data = json.dumps(body_params)
    _assert_collaborator_added(test_folder, invitee, mock_box_session, mock_collab_response, notify, role, can_view_path, data)


def test_add_collaborator_raises_for_bad_type(test_folder):
    with pytest.raises(TypeError):
        test_folder.add_collaborator(b'byte string', CollaborationRole.EDITOR)


@pytest.mark.parametrize('recursive', [True, False])
def test_delete_folder(test_folder, mock_box_session, recursive, etag, if_match_header):
    test_folder.delete(recursive=recursive, etag=etag)
    expected_url = test_folder.get_url()
    mock_box_session.delete.assert_called_once_with(
        expected_url,
        expect_json_response=False,
        params={'recursive': recursive},
        headers=if_match_header,
    )


@pytest.mark.parametrize('limit,offset,fields', [(1, 0, None), (100, 0, ['foo', 'bar']), (1, 1, None)])
def test_get_items(test_folder, mock_box_session, mock_items_response, limit, offset, fields):
    # pylint:disable=redefined-outer-name
    expected_url = test_folder.get_url('items')
    mock_box_session.get.return_value, expected_items = mock_items_response(limit, offset)
    items = test_folder.get_items(limit, offset, fields)
    expected_params = {'limit': limit, 'offset': offset}
    if fields:
        expected_params['fields'] = ','.join(fields)
    mock_box_session.get.assert_called_once_with(expected_url, params=expected_params)
    assert items == expected_items
    assert all([i.id == e.object_id for i, e in zip(items, expected_items)])


def test_get_items_marker_returns_marker_instance(test_folder):
    limit_offset_object_collection = test_folder.get_items_limit_offset()
    assert isinstance(limit_offset_object_collection, LimitOffsetBasedObjectCollection)


def test_get_items_limit_offset_returns_limit_offset_instance(test_folder):
    marker_object_collection = test_folder.get_items_marker()
    assert isinstance(marker_object_collection, MarkerBasedObjectCollection)


@pytest.mark.parametrize('is_stream', (True, False))
def test_upload(
        test_folder,
        mock_box_session,
        mock_content_response,
        mock_upload_response,
        mock_file_path,
        mock_object_id,
        upload_using_accelerator,
        mock_accelerator_response_for_new_uploads,
        mock_new_upload_accelerator_url,
        upload_using_accelerator_fails,
        is_stream,
):
    expected_url = '{0}/files/content'.format(API.UPLOAD_URL)
    if upload_using_accelerator:
        if upload_using_accelerator_fails:
            mock_box_session.options.side_effect = BoxAPIException(400)
        else:
            mock_box_session.options.return_value = mock_accelerator_response_for_new_uploads
            expected_url = mock_new_upload_accelerator_url

    mock_box_session.post.return_value = mock_upload_response

    if is_stream:
        mock_file_stream = BytesIO(mock_content_response.content)
        new_file = test_folder.upload_stream(
            mock_file_stream,
            basename(mock_file_path),
            upload_using_accelerator=upload_using_accelerator,
        )
    else:
        mock_file = mock_open(read_data=mock_content_response.content)
        mock_file_stream = mock_file.return_value
        with patch('boxsdk.object.folder.open', mock_file, create=True):
            new_file = test_folder.upload(
                mock_file_path,
                upload_using_accelerator=upload_using_accelerator,
            )

    mock_files = {'file': ('unused', mock_file_stream)}
    data = {'attributes': json.dumps({'name': basename(mock_file_path), 'parent': {'id': mock_object_id}})}
    mock_box_session.post.assert_called_once_with(expected_url, expect_json_response=False, files=mock_files, data=data)
    assert isinstance(new_file, File)
    assert new_file.object_id == mock_object_id
    assert 'id' in new_file
    assert new_file['id'] == mock_object_id
    assert not hasattr(new_file, 'entries')
    assert 'entries' not in new_file


def test_upload_stream_does_preflight_check_if_specified(
        mock_box_session,
        test_folder,
        preflight_check,
        preflight_fails,
        file_size,
):
    with patch.object(Folder, 'preflight_check', return_value=None):
        kwargs = {'file_stream': BytesIO(b'some bytes'), 'file_name': 'foo.txt'}
        mock_box_session.post = MagicMock()
        if preflight_check:
            kwargs['preflight_check'] = preflight_check
            kwargs['preflight_expected_size'] = file_size
        if preflight_fails:
            test_folder.preflight_check.side_effect = BoxAPIException(400)
            with pytest.raises(BoxAPIException):
                test_folder.upload_stream(**kwargs)
        else:
            test_folder.upload_stream(**kwargs)

        if preflight_check:
            assert test_folder.preflight_check.called_once_with(size=file_size, name='foo.txt')
            _assert_post_called_correctly(mock_box_session, preflight_fails)
        else:
            assert not test_folder.preflight_check.called


def _assert_post_called_correctly(mock_box_session, preflight_fails):
    if preflight_fails:
        assert not mock_box_session.post.called
    else:
        assert mock_box_session.post.called


@patch('boxsdk.object.folder.open', mock_open(read_data=b'some bytes'), create=True)
def test_upload_does_preflight_check_if_specified(
        mock_box_session,
        test_folder,
        mock_file_path,
        preflight_check,
        preflight_fails,
        file_size,
):
    with patch.object(Folder, 'preflight_check', return_value=None):
        kwargs = {'file_path': mock_file_path, 'file_name': 'foo.txt'}
        mock_box_session.post = MagicMock()
        if preflight_check:
            kwargs['preflight_check'] = preflight_check
            kwargs['preflight_expected_size'] = file_size
        if preflight_fails:
            test_folder.preflight_check.side_effect = BoxAPIException(400)
            with pytest.raises(BoxAPIException):
                test_folder.upload(**kwargs)
        else:
            test_folder.upload(**kwargs)

        if preflight_check:
            assert test_folder.preflight_check.called_once_with(size=file_size, name='foo.txt')
            _assert_post_called_correctly(mock_box_session, preflight_fails)
        else:
            assert not test_folder.preflight_check.called


def test_create_subfolder(test_folder, mock_box_session, mock_object_id, mock_folder_response):
    expected_url = test_folder.get_type_url()
    mock_box_session.post.return_value = mock_folder_response
    new_folder = test_folder.create_subfolder('name')
    data = json.dumps({'name': 'name', 'parent': {'id': mock_object_id}})
    mock_box_session.post.assert_called_once_with(expected_url, data=data)
    assert isinstance(new_folder, Folder)
    assert new_folder.object_id == mock_object_id


@pytest.mark.parametrize('sync_state', iter(FolderSyncState))
def test_update_sync_state(test_folder, mock_folder_response, mock_box_session, sync_state):
    expected_url = test_folder.get_url()
    mock_box_session.put.return_value = mock_folder_response
    data = {'sync_state': sync_state}
    update_response = test_folder.update_sync_state(sync_state)
    mock_box_session.put.assert_called_once_with(expected_url, data=json.dumps(data), params=None, headers=None)
    assert isinstance(update_response, Folder)
    assert update_response.object_id == test_folder.object_id


def test_preflight(test_folder, mock_object_id, mock_box_session):
    new_file_size, new_file_name = 100, 'foo.txt'
    test_folder.preflight_check(size=new_file_size, name=new_file_name)
    mock_box_session.options.assert_called_once_with(
        url='{0}/files/content'.format(API.BASE_API_URL),
        expect_json_response=False,
        data=json.dumps(
            {
                'size': new_file_size,
                'name': new_file_name,
                'parent': {'id': mock_object_id},
            }
        ),
    )



# coding: utf-8

from __future__ import absolute_import, unicode_literals

from functools import partial
from io import IOBase
from numbers import Number

from mock import MagicMock, Mock, PropertyMock, call
import pytest

from boxsdk.auth.oauth2 import OAuth2
from boxsdk.exception import BoxAPIException
from boxsdk.network.default_network import DefaultNetwork, DefaultNetworkResponse
from boxsdk.session.box_session import BoxSession, BoxResponse, Translator


@pytest.fixture(scope='function', params=[False, True])
def translator(default_translator, request):  # pylint:disable=unused-argument
    if request.param:
        return Translator(extend_default_translator=True, new_child=True)
    return None


@pytest.fixture
def initial_access_token():
    return 'fake_access_token'


@pytest.fixture
def mock_oauth(initial_access_token):
    mock_oauth = MagicMock(OAuth2)
    mock_oauth.access_token = initial_access_token
    return mock_oauth


@pytest.fixture
def mock_network_layer():
    return Mock(DefaultNetwork)


@pytest.fixture
def box_session(mock_oauth, mock_network_layer, translator):
    # pylint:disable=redefined-outer-name
    return BoxSession(oauth=mock_oauth, network_layer=mock_network_layer, translator=translator)


@pytest.mark.parametrize('test_method', [
    BoxSession.get,
    BoxSession.post,
    BoxSession.put,
    BoxSession.delete,
    BoxSession.options,
])
def test_box_session_handles_unauthorized_response(
        test_method,
        box_session,
        mock_oauth,
        mock_network_layer,
        unauthorized_response,
        generic_successful_response,
        test_url,
):
    # pylint:disable=redefined-outer-name

    def get_access_token_from_auth_object():
        return mock_oauth.access_token

    mock_network_layer.request.side_effect = mock_responses = [unauthorized_response, generic_successful_response]
    for mock_response in mock_responses:
        type(mock_response).access_token_used = PropertyMock(side_effect=get_access_token_from_auth_object)

    def refresh(access_token_used):
        assert access_token_used == mock_oauth.access_token
        mock_oauth.access_token = 'fake_new_access_token'
        return (mock_oauth.access_token, None)

    mock_oauth.refresh.side_effect = refresh

    box_response = test_method(box_session, url=test_url)
    assert box_response.status_code == 200


@pytest.mark.parametrize('test_method', [
    BoxSession.get,
    BoxSession.post,
    BoxSession.put,
    BoxSession.delete,
    BoxSession.options,
])
@pytest.mark.parametrize('initial_access_token', [None])
def test_box_session_gets_access_token_before_request(
        test_method,
        box_session,
        mock_oauth,
        mock_network_layer,
        generic_successful_response,
        test_url,
):
    # pylint:disable=redefined-outer-name

    def get_access_token_from_auth_object():
        return mock_oauth.access_token

    mock_network_layer.request.side_effect = mock_responses = [generic_successful_response]
    for mock_response in mock_responses:
        type(mock_response).access_token_used = PropertyMock(side_effect=get_access_token_from_auth_object)

    def refresh(access_token_used):
        assert access_token_used == mock_oauth.access_token
        mock_oauth.access_token = 'fake_new_access_token'
        return (mock_oauth.access_token, None)

    mock_oauth.refresh.side_effect = refresh

    box_response = test_method(box_session, url=test_url, auto_session_renewal=True)
    assert box_response.status_code == 200


@pytest.mark.parametrize('test_method', [
    BoxSession.get,
    BoxSession.post,
    BoxSession.put,
    BoxSession.delete,
    BoxSession.options,
    partial(BoxSession.request, method='head'),
])
def test_box_session_retries_response_after_retry_after(
        test_method,
        box_session,
        mock_network_layer,
        retry_after_response,
        generic_successful_response,
        test_url,
):
    # pylint:disable=redefined-outer-name
    mock_network_layer.request.side_effect = [retry_after_response, generic_successful_response]
    mock_network_layer.retry_after.side_effect = lambda delay, request, *args, **kwargs: request(*args, **kwargs)

    box_response = test_method(box_session, url=test_url)
    assert box_response.status_code == 200
    assert len(mock_network_layer.retry_after.call_args_list) == 1
    assert isinstance(mock_network_layer.retry_after.call_args[0][0], Number)
    assert mock_network_layer.retry_after.call_args[0][0] == float(retry_after_response.headers['Retry-After'])


@pytest.mark.parametrize('test_method', [
    BoxSession.get,
    BoxSession.post,
    BoxSession.put,
    BoxSession.delete,
    BoxSession.options,
    partial(BoxSession.request, method='head'),
])
def test_box_session_retries_request_after_server_error(
        test_method,
        box_session,
        mock_network_layer,
        server_error_response,
        generic_successful_response,
        test_url,
):
    # pylint:disable=redefined-outer-name
    mock_network_layer.request.side_effect = [server_error_response, server_error_response, generic_successful_response]
    mock_network_layer.retry_after.side_effect = lambda delay, request, *args, **kwargs: request(*args, **kwargs)

    box_response = test_method(box_session, url=test_url)
    assert box_response.status_code == 200
    assert box_response.json() == generic_successful_response.json()
    assert box_response.ok == generic_successful_response.ok
    assert box_response.content == generic_successful_response.content
    assert len(mock_network_layer.retry_after.call_args_list) == 2
    assert isinstance(mock_network_layer.retry_after.call_args_list[0][0][0], Number)
    assert isinstance(mock_network_layer.retry_after.call_args_list[1][0][0], Number)
    assert mock_network_layer.retry_after.call_args_list[0][0][0] == 1
    assert mock_network_layer.retry_after.call_args_list[1][0][0] == 2


def test_box_session_seeks_file_after_retry(box_session, mock_network_layer, server_error_response, generic_successful_response, test_url):
    # pylint:disable=redefined-outer-name
    mock_network_layer.request.side_effect = [server_error_response, generic_successful_response]
    mock_network_layer.retry_after.side_effect = lambda delay, request, *args, **kwargs: request(*args, **kwargs)
    mock_file_1, mock_file_2 = MagicMock(IOBase), MagicMock(IOBase)
    mock_file_1.tell.return_value = 0
    mock_file_2.tell.return_value = 3
    files = {'file': ('unused', mock_file_1), 'f2': ('unused', mock_file_2)}

    box_response = box_session.post(url=test_url, files=files)
    assert box_response.status_code == 200
    assert box_response.json() == generic_successful_response.json()
    assert box_response.ok == generic_successful_response.ok
    mock_file_1.tell.assert_called_with()
    mock_file_2.tell.assert_called_with()
    mock_file_1.seek.assert_called_with(0)
    assert mock_file_1.seek.call_count == 2
    assert mock_file_1.seek.has_calls(call(0) * 2)
    mock_file_2.seek.assert_called_with(3)
    assert mock_file_2.seek.call_count == 2
    assert mock_file_2.seek.has_calls(call(3) * 2)


def test_box_session_raises_for_non_json_response(box_session, mock_network_layer, non_json_response, test_url):
    # pylint:disable=redefined-outer-name
    mock_network_layer.request.side_effect = [non_json_response]

    with pytest.raises(BoxAPIException):
        box_session.get(url=test_url)


def test_box_session_raises_for_failed_response(box_session, mock_network_layer, bad_network_response, test_url):
    # pylint:disable=redefined-outer-name
    mock_network_layer.request.side_effect = [bad_network_response]

    with pytest.raises(BoxAPIException):
        box_session.get(url=test_url)


def test_box_session_raises_for_failed_non_json_response(box_session, mock_network_layer, failed_non_json_response, test_url):
    # pylint:disable=redefined-outer-name
    mock_network_layer.request.side_effect = [failed_non_json_response]

    with pytest.raises(BoxAPIException):
        box_session.get(url=test_url, expect_json_response=False)


def test_box_response_properties_pass_through_to_network_response_properties():
    mock_network_response = Mock(DefaultNetworkResponse)
    box_result = BoxResponse(mock_network_response)
    assert box_result.json() == mock_network_response.json()
    assert box_result.content == mock_network_response.content
    assert box_result.ok == mock_network_response.ok
    assert box_result.status_code == mock_network_response.status_code
    assert box_result.network_response == mock_network_response


def test_translator(box_session, translator, default_translator, original_default_translator):
    assert isinstance(box_session.translator, Translator)
    assert box_session.translator == default_translator
    if translator:
        assert box_session.translator is translator

    # Test that adding new registrations works.

    class Foo(object):
        pass

    item_type = u'ƒøø'
    box_session.translator.register(item_type, Foo)
    assert box_session.translator.translate(item_type) is Foo

    # Test that adding new registrations does not affect global state.
    assert default_translator == original_default_translator
    assert (set(box_session.translator) - set(default_translator)) == set([item_type])



# coding: utf-8

from __future__ import unicode_literals
from boxsdk.util.text_enum import TextEnum


class MockTextEnum(TextEnum):
    member = 'member'


def test_text_enum_repr_is_value():
    assert MockTextEnum.member.__repr__() == MockTextEnum.member.value  # pylint:disable=no-member


def test_text_enum_str_is_value():
    assert str(MockTextEnum.member) == str('member')



# coding: utf-8

from __future__ import unicode_literals
from mock import call, patch
import pytest
from boxsdk.config import API, Client
from boxsdk.object.group_membership import GroupMembership
from boxsdk.util.shared_link import get_shared_link_header


@pytest.fixture
def shared_link():
    return 'https://app.box.com/s/q2i1024dvguiads6mzj2avsq9hmz43du'


@pytest.fixture(params=(None, 'shared_link_password'))
def shared_link_password(request):
    return request.param


@pytest.fixture
def box_api_headers(shared_link, shared_link_password, access_token):
    # pylint:disable=redefined-outer-name
    box_api_header = get_shared_link_header(shared_link, shared_link_password)['BoxApi']
    return {
        'Authorization': 'Bearer {0}'.format(access_token),
        'BoxApi': box_api_header,
        'User-Agent': Client.USER_AGENT_STRING,
    }


def test_client_with_shared_link_causes_box_api_header_to_be_added(
        box_client,
        mock_box_network,
        generic_successful_response,
        shared_link,
        shared_link_password,
        box_api_headers,
):
    # pylint:disable=redefined-outer-name
    mock_box_network.session.request.side_effect = [
        generic_successful_response,
    ]
    box_client.with_shared_link(shared_link, shared_link_password).folder('0').get()
    assert mock_box_network.session.request.mock_calls == [
        call(
            'GET',
            '{0}/folders/0'.format(API.BASE_API_URL),
            headers=box_api_headers,
            params=None,
        ),
    ]


def test_folder_object_with_shared_link_causes_box_api_header_to_be_added(
        box_client,
        mock_box_network,
        generic_successful_response,
        shared_link,
        shared_link_password,
        box_api_headers,
):
    # pylint:disable=redefined-outer-name
    mock_box_network.session.request.side_effect = [
        generic_successful_response,
    ]
    box_client.folder('0').with_shared_link(shared_link, shared_link_password).get()
    assert mock_box_network.session.request.mock_calls == [
        call(
            'GET',
            '{0}/folders/0'.format(API.BASE_API_URL),
            headers=box_api_headers,
            params=None,
        ),
    ]


def test_group_membership_object_with_shared_link_causes_box_api_header_to_be_added(
        box_client,
        mock_box_network,
        generic_successful_response,
        shared_link,
        shared_link_password,
        box_api_headers,
):
    # pylint:disable=redefined-outer-name
    mock_box_network.session.request.side_effect = [
        generic_successful_response,
    ]
    with patch.object(GroupMembership, '_init_user_and_group_instances') as init:
        init.return_value = None, None
        box_client.group_membership('0').with_shared_link(shared_link, shared_link_password).get()
    assert mock_box_network.session.request.mock_calls == [
        call(
            'GET',
            '{0}/group_memberships/0'.format(API.BASE_API_URL),
            headers=box_api_headers,
            params=None,
        ),
    ]


def test_events_endpoint_with_shared_link_causes_box_api_header_to_be_added(
        box_client,
        mock_box_network,
        generic_successful_response,
        shared_link,
        shared_link_password,
        box_api_headers,
):
    # pylint:disable=redefined-outer-name
    mock_box_network.session.request.side_effect = [
        generic_successful_response,
    ]
    stream_position = 1348790499819
    options = {'url': '{0}/events'.format(API.BASE_API_URL), 'retry_timeout': 60}
    box_client.events().with_shared_link(shared_link, shared_link_password).long_poll(options, stream_position)
    assert mock_box_network.session.request.mock_calls == [
        call(
            'GET',
            options['url'],
            headers=box_api_headers,
            timeout=options['retry_timeout'],
            params={'stream_position': stream_position},
        ),
    ]


def test_metadata_endpoint_with_shared_link_causes_box_api_header_to_be_added(
        box_client,
        mock_box_network,
        generic_successful_response,
        shared_link,
        shared_link_password,
        box_api_headers,
):
    # pylint:disable=redefined-outer-name
    mock_box_network.session.request.side_effect = [
        generic_successful_response,
    ]
    box_client.file('0').metadata().with_shared_link(shared_link, shared_link_password).get()
    assert mock_box_network.session.request.mock_calls == [
        call(
            'GET',
            '{0}/files/0/metadata/global/properties'.format(API.BASE_API_URL),
            headers=box_api_headers,
        ),
    ]



# coding: utf-8

from __future__ import unicode_literals
from datetime import datetime
import sqlalchemy
import uuid
from test.functional.mock_box.db_model import DbModel


class EventModel(DbModel):
    """DB Model for Box events."""
    __tablename__ = 'box_event'

    id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True, autoincrement=True)  # pylint:disable=invalid-name
    event_id = sqlalchemy.Column(sqlalchemy.String(32), default=lambda: uuid.uuid4().hex)
    stream_position = sqlalchemy.Column(sqlalchemy.DateTime, default=datetime.utcnow)
    created_by_id = sqlalchemy.Column(sqlalchemy.Integer, sqlalchemy.ForeignKey('box_user.id'))
    event_type = sqlalchemy.Column(sqlalchemy.Enum(
        'ITEM_CREATE',
        'ITEM_UPLOAD',
        'ITEM_MOVE',
        'ITEM_COPY',
        'LOCK_CREATE',
        'LOCK_DESTROY',
        'ITEM_TRASH',
        'ITEM_UNDELETE_VIA_TRASH',
        'COLLAB_ADD_COLLABORATOR',
        'COLLAB_REMOVE_COLLABORATOR',
        'ITEM_SYNC',
        'ITEM_UNSYNC',
        'ITEM_RENAME',
    ))
    source_id = sqlalchemy.Column(sqlalchemy.Integer)
    source_type = sqlalchemy.Column(sqlalchemy.Enum('file', 'folder'))
    sequence_id = sqlalchemy.Column(sqlalchemy.Integer, nullable=False)

    __mapper_args__ = {
        "version_id_col": sequence_id,
    }



# coding: utf-8

from __future__ import unicode_literals
import sqlalchemy
from sqlalchemy.orm import relationship
from test.functional.mock_box.db_model import DbModel


_user_app_table = sqlalchemy.Table(
    'user_app_association',
    DbModel.metadata,
    sqlalchemy.Column('app_id', sqlalchemy.Integer, sqlalchemy.ForeignKey('box_application.id')),
    sqlalchemy.Column('user_id', sqlalchemy.Integer, sqlalchemy.ForeignKey('box_user.id'))
)


class ApplicationModel(DbModel):
    """DB Model for Box users."""
    __tablename__ = 'box_application'

    id = sqlalchemy.Column(sqlalchemy.Integer, primary_key=True, autoincrement=True)  # pylint:disable=invalid-name
    client_id = sqlalchemy.Column(sqlalchemy.String(32), nullable=False)
    client_secret = sqlalchemy.Column(sqlalchemy.String(32), nullable=False)
    auth_tokens = relationship('TokenModel', backref='application', cascade='save-update, delete')
    users = relationship('UserModel', secondary=_user_app_table, backref='apps')



# coding: utf-8

from __future__ import absolute_import, unicode_literals

import json
import logging
import sys

from mock import Mock
import pytest
import requests
from six import binary_type

from boxsdk.network.default_network import DefaultNetworkResponse


@pytest.fixture(autouse=True, scope='session')
def logger():
    logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
    return logging.getLogger(__name__.split('.')[0])


def _set_content_and_json_from_json(mock_response, json_value):
    mock_response.json.return_value = json_value
    mock_response.content = content = json.dumps(json_value).encode('utf-8')
    mock_response.headers['Content-Length'] = str(len(content))


def _set_content_and_json_from_content(mock_response, content):
    if not isinstance(content, binary_type):
        raise TypeError("Expected 'content' to be byte string, got {!r}.".format(content.__class__.__name__))
    mock_response.content = content
    mock_response.headers['Content-Length'] = str(len(content))
    try:
        mock_response.json.return_value = json.loads(content.decode('utf-8'))
    except ValueError as exc:
        mock_response.json.side_effect = exc


@pytest.fixture()
def generic_successful_request_response():
    mock_request_response = Mock(requests.Response(), headers=dict([('header{0}'.format(i), 'value{0}'.format(i)) for i in range(4)]))
    _set_content_and_json_from_json(mock_request_response, json_value=dict([('key{0}'.format(i), 'value{0}'.format(i)) for i in range(8)]))
    mock_request_response.status_code = 200
    mock_request_response.ok = True
    return mock_request_response


def _network_response_mock_from_request_response(request_response):
    mock_network_response = Mock(DefaultNetworkResponse)
    mock_network_response.request_response = request_response
    mock_network_response.json.side_effect = request_response.json
    mock_network_response.content = request_response.content
    mock_network_response.headers = request_response.headers
    mock_network_response.status_code = request_response.status_code
    mock_network_response.ok = request_response.ok
    return mock_network_response


@pytest.fixture()
def generic_successful_response(generic_successful_request_response):
    return _network_response_mock_from_request_response(generic_successful_request_response)


@pytest.fixture(scope='session')
def successful_token_json_response(access_token, refresh_token):
    # pylint:disable=redefined-outer-name
    return {
        'access_token': access_token,
        'expires_in': 3600,
        'restricted_to': [],
        'token_type': 'bearer',
        'refresh_token': refresh_token,
    }


@pytest.fixture()
def successful_token_request_response(successful_token_json_response):
    # pylint:disable=redefined-outer-name
    successful_token_mock = Mock(requests.Response(), headers={})
    _set_content_and_json_from_json(successful_token_mock, json_value=successful_token_json_response)
    successful_token_mock.ok = True
    successful_token_mock.status_code = 200
    return successful_token_mock


@pytest.fixture()
def successful_token_response(successful_token_request_response):
    return _network_response_mock_from_request_response(successful_token_request_response)


@pytest.fixture()
def successful_token_mock(successful_token_response):
    return successful_token_response


@pytest.fixture()
def unauthorized_response():
    res = Mock(DefaultNetworkResponse, headers={})
    _set_content_and_json_from_content(res, content=b'')
    res.status_code = 401
    res.ok = False
    return res


@pytest.fixture()
def non_json_response():
    mock_network_response = Mock(DefaultNetworkResponse, headers={})
    _set_content_and_json_from_content(mock_network_response, content=b'')
    mock_network_response.ok = True
    mock_network_response.status_code = 200
    return mock_network_response


def _retry_after_response(status_code):
    mock_network_response = Mock(DefaultNetworkResponse, headers={})
    mock_network_response.status_code = status_code
    mock_network_response.headers.update({'Retry-After': '1'})
    return mock_network_response


@pytest.fixture()
def retry_after_response_202():
    return _retry_after_response(202)


@pytest.fixture()
def retry_after_response_429():
    return _retry_after_response(429)


@pytest.fixture(params=[202, 429])
def retry_after_response(retry_after_response_202, retry_after_response_429, request):
    if request.param == 202:
        return retry_after_response_202
    elif request.param == 429:
        return retry_after_response_429
    else:
        raise ValueError


def _server_error_request_response(status_code):
    mock_request_response = Mock(requests.Response(), headers=dict([('header{0}'.format(i), 'value{0}'.format(i)) for i in range(4)]))
    _set_content_and_json_from_json(mock_request_response, json_value=dict([('key{0}'.format(i), 'value{0}'.format(i)) for i in range(8)]))
    mock_request_response.status_code = status_code
    mock_request_response.ok = False
    return mock_request_response


@pytest.fixture()
def server_error_request_response_502():
    return _server_error_request_response(502)


@pytest.fixture()
def server_error_request_response_503():
    return _server_error_request_response(503)


@pytest.fixture(params=[502, 503])
def server_error_request_response(server_error_request_response_502, server_error_request_response_503, request):
    if request.param == 502:
        return server_error_request_response_502
    elif request.param == 503:
        return server_error_request_response_503
    else:
        raise ValueError


@pytest.fixture
def server_error_response(server_error_request_response):
    return _network_response_mock_from_request_response(server_error_request_response)


@pytest.fixture()
def bad_network_response():
    mock_network_response = Mock(DefaultNetworkResponse, headers={})
    mock_network_response.status_code = 404
    _set_content_and_json_from_json(mock_network_response, json_value={'code': 404, 'message': 'Not Found'})
    mock_network_response.ok = False
    return mock_network_response


@pytest.fixture()
def failed_non_json_response():
    mock_network_response = Mock(DefaultNetworkResponse, headers={})
    mock_network_response.status_code = 404
    _set_content_and_json_from_content(mock_network_response, content=b'')
    mock_network_response.ok = False
    return mock_network_response


@pytest.fixture(scope='session')
def access_token():
    return 'T9cE5asGnuyYCCqIZFoWjFHvNbvVqHjl'


@pytest.fixture(scope='session')
def new_access_token():
    # Must be distinct from access_token.
    return 'ZFoWjFHvNbvVqHjlT9cE5asGnuyYCCqI'


@pytest.fixture(scope='session')
def refresh_token():
    return 'J7rxTiWOHMoSC1isKZKBZWizoRXjkQzig5C6jFgCVJ9bUnsUfGMinKBDLZWP9BgRb'


@pytest.fixture(scope='session')
def test_url():
    return 'https://box.com/test/url'


@pytest.fixture(scope='session')
def client_id():
    return 'fake_client_id'


@pytest.fixture(scope='session')
def client_secret():
    return 'fake_client_secret'


@pytest.fixture(scope='session')
def auth_code():
    return 'fake_auth_code'


@pytest.fixture(params=[
    b'Hello',
    b'Goodbye',
    b'42',
])
def test_file_content(request):
    return request.param


@pytest.fixture()
def update_file_content(test_file_content):
    # pylint:disable=redefined-outer-name
    return test_file_content


@pytest.fixture()
def test_file_path():
    return 'path/to/file'


@pytest.fixture(scope='module')
def mock_object_id():
    return '42'


@pytest.fixture(scope='module')
def mock_user_id():
    return 'fake-user-100'


@pytest.fixture(scope='module')
def mock_group_id():
    return 'fake-group-99'



# coding: utf-8

from __future__ import unicode_literals

from codecs import open   # pylint:disable=redefined-builtin
from collections import defaultdict
from os.path import dirname, join
import re
import sys

from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand


CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: Apache Software License',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: Implementation :: CPython',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Operating System :: OS Independent',
    'Operating System :: POSIX',
    'Operating System :: Microsoft :: Windows',
    'Operating System :: MacOS :: MacOS X',
    'Topic :: Software Development :: Libraries :: Python Modules',
]


class PyTest(TestCommand):
    # pylint:disable=attribute-defined-outside-init

    user_options = [(b'pytest-args=', b'a', b"Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = None

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # Do the import here, once the eggs are loaded.
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


def main():
    base_dir = dirname(__file__)
    install_requires = ['requests>=2.4.3', 'six>=1.9.0', 'requests-toolbelt>=0.4.0']
    redis_requires = ['redis>=2.10.3']
    jwt_requires = ['pyjwt>=1.3.0', 'cryptography>=0.9.2']
    extra_requires = defaultdict(list)
    extra_requires.update({'jwt': jwt_requires, 'redis': redis_requires, 'all': jwt_requires + redis_requires})
    conditional_dependencies = {
        # Newer versions of pip and wheel, which support PEP 426, allow
        # environment markers for conditional dependencies to use operators
        # such as `<` and `<=` [1]. However, older versions of pip and wheel
        # only support PEP 345, which only allows operators `==` and `in` (and
        # their negations) along with string constants [2]. To get the widest
        # range of support, we'll only use the `==` operator, which means
        # explicitly listing all supported Python versions that need the extra
        # dependencies.
        #
        # [1] <https://www.python.org/dev/peps/pep-0426/#environment-markers>
        # [2] <https://www.python.org/dev/peps/pep-0345/#environment-markers>
        'chainmap>=1.0.2': ['2.6', '2.7'],  # <'3.3'
        'enum34>=1.0.4': ['2.6', '2.7', '3.3'],   # <'3.4'
        'ordereddict>=1.1': ['2.6'],   # <'2.7'
    }
    for requirement, python_versions in conditional_dependencies.items():
        for python_version in python_versions:
            # <https://wheel.readthedocs.org/en/latest/#defining-conditional-dependencies>
            python_conditional = 'python_version=="{0}"'.format(python_version)
            key = ':{0}'.format(python_conditional)
            extra_requires[key].append(requirement)
    with open('boxsdk/version.py', 'r', encoding='utf-8') as config_py:
        version = re.search(r'^\s+__version__\s*=\s*[\'"]([^\'"]*)[\'"]', config_py.read(), re.MULTILINE).group(1)
    setup(
        name='boxsdk',
        version=version,
        description='Official Box Python SDK',
        long_description=open(join(base_dir, 'README.rst'), encoding='utf-8').read(),
        author='Box',
        author_email='oss@box.com',
        url='http://opensource.box.com',
        packages=find_packages(exclude=['demo', 'docs', 'test', 'test*', '*test', '*test*']),
        install_requires=install_requires,
        extras_require=extra_requires,
        tests_require=['pytest', 'pytest-xdist', 'mock', 'sqlalchemy', 'bottle', 'jsonpatch'],
        cmdclass={'test': PyTest},
        classifiers=CLASSIFIERS,
        keywords='box oauth2 sdk',
        license='Apache Software License, Version 2.0, http://www.apache.org/licenses/LICENSE-2.0',
    )


if __name__ == '__main__':
    main()



# coding: utf-8

from __future__ import unicode_literals, absolute_import

from mock import Mock

from boxsdk.auth import remote_managed_oauth2


def test_remote_managed_oauth2_calls_retrieve_tokens_during_refresh(access_token):
    retrieve_access_token = Mock()
    oauth2 = remote_managed_oauth2.RemoteOAuth2(
        retrieve_access_token=retrieve_access_token,
        client_id=None,
        client_secret=None,
        access_token=access_token,
    )
    retrieve_access_token.return_value = access_token
    assert oauth2.refresh(access_token) == (access_token, None)
    retrieve_access_token.assert_called_once_with(access_token)



# coding: utf-8

from __future__ import absolute_import, unicode_literals

from enum import Enum
import pytest

from boxsdk.util.compat import with_metaclass
from boxsdk.util.enum import ExtendableEnumMeta
from boxsdk.util.ordered_dict import OrderedDict


# pylint:disable=invalid-name
# So that we can have class definitions as pytest function fixtures.


@pytest.fixture(scope='function')
def EnumBase():

    class EnumBase(with_metaclass(ExtendableEnumMeta, Enum)):
        pass

    return EnumBase


@pytest.fixture(scope='function')
def Enum1(EnumBase):

    class Enum1(EnumBase):
        A = 1

    return Enum1


@pytest.fixture(scope='function')
def Enum2(EnumBase):

    class Enum2(EnumBase):
        pass

    return Enum2


@pytest.fixture(scope='function')
def Enum2_1(Enum2):

    class Enum2_1(Enum2):
        B = 2

    return Enum2_1


@pytest.fixture(scope='function')
def Enum2_2(Enum2):

    class Enum2_2(Enum2):
        C = 3

    return Enum2_2


@pytest.fixture(scope='function')
def EnumBaseWithSubclassesDefined(EnumBase, Enum1, Enum2_1, Enum2_2):   # pylint:disable=unused-argument
    return EnumBase


enum_member_names = ['A', 'B', 'C']
enum_member_values = [1, 2, 3]


@pytest.fixture(scope='session', params=enum_member_names)
def enum_member_name(request):
    return request.param


@pytest.fixture(scope='session', params=enum_member_values)
def enum_member_value(request):
    return request.param


@pytest.fixture(scope='function')
def enum_members(Enum1, Enum2_1, Enum2_2):
    members = OrderedDict()
    for enum_member_name, enum_class in zip(enum_member_names, [Enum1, Enum2_1, Enum2_2]):
        members[enum_member_name] = enum_class[enum_member_name]
    return members


@pytest.fixture(scope='function')
def enum_instance(enum_member_name, enum_members):
    return enum_members[enum_member_name]


def test_can_construct_enum_hierarchy(EnumBaseWithSubclassesDefined):   # pylint:disable=unused-argument
    pass


def test_can_not_construct_non_leaf_enum_with_members(Enum1):
    with pytest.raises(TypeError):
        class Enum1_1(Enum1):   # pylint:disable=unused-variable
            pass


def test_lookup(EnumBaseWithSubclassesDefined, enum_member_value):
    EnumBase = EnumBaseWithSubclassesDefined
    enum_instance = EnumBase.lookup(enum_member_value)
    assert isinstance(enum_instance, EnumBase)
    assert EnumBase.lookup(enum_instance) == enum_instance
    assert enum_instance.__class__.lookup(enum_instance) == enum_instance
    assert enum_instance.__class__(enum_instance) == enum_instance
    assert enum_instance.__class__(enum_member_value) == enum_instance


def test_lookup_raises_value_error_for_non_members(EnumBaseWithSubclassesDefined):
    with pytest.raises(ValueError):
        EnumBaseWithSubclassesDefined.lookup('foobar')


def test_members(EnumBaseWithSubclassesDefined, enum_members):
    assert dict(EnumBaseWithSubclassesDefined.__members__) == enum_members


def test_contains_enum_instances(EnumBaseWithSubclassesDefined, enum_instance):
    assert enum_instance in EnumBaseWithSubclassesDefined


def test_contains_returns_false_for_non_instances(EnumBaseWithSubclassesDefined, enum_member_name):
    assert enum_member_name not in EnumBaseWithSubclassesDefined


def test_getitem(EnumBaseWithSubclassesDefined, enum_member_name, enum_instance):
    assert EnumBaseWithSubclassesDefined[enum_member_name] == enum_instance


def test_getitem_raises_key_error_for_non_member_names(EnumBaseWithSubclassesDefined, enum_member_value):
    with pytest.raises(KeyError):
        EnumBaseWithSubclassesDefined[enum_member_value]  # pylint:disable=pointless-statement


def test_getattr(EnumBaseWithSubclassesDefined, enum_member_name, enum_instance):
    assert getattr(EnumBaseWithSubclassesDefined, enum_member_name) == enum_instance


def test_getattr_raises_attribute_error_for_non_member_names(EnumBaseWithSubclassesDefined):
    with pytest.raises(AttributeError):
        EnumBaseWithSubclassesDefined.foobar  # pylint:disable=pointless-statement


def test_getattr_does_get_arbitrary_attributes_from_itself(EnumBaseWithSubclassesDefined):
    EnumBase = EnumBaseWithSubclassesDefined
    EnumBase.foobar = 'foobar'
    assert EnumBase.foobar == 'foobar'


def test_getattr_does_not_get_arbitrary_attributes_from_subclasses(EnumBaseWithSubclassesDefined, Enum1):
    Enum1.foobar = 'foobar'
    with pytest.raises(AttributeError):
        EnumBaseWithSubclassesDefined.foobar  # pylint:disable=pointless-statement


def test_iter(EnumBaseWithSubclassesDefined, enum_members):
    assert list(EnumBaseWithSubclassesDefined) == list(enum_members.values())


def test_len(EnumBaseWithSubclassesDefined, enum_members):
    assert len(EnumBaseWithSubclassesDefined) == len(enum_members)


def test_reversed(EnumBaseWithSubclassesDefined):
    EnumBase = EnumBaseWithSubclassesDefined
    assert list(reversed(list(reversed(EnumBase)))) == list(EnumBase)


def test_dir(EnumBaseWithSubclassesDefined):
    assert set(enum_member_names).issubset(dir(EnumBaseWithSubclassesDefined))



# coding: utf-8

from __future__ import unicode_literals
import json
import os
from six import text_type

from boxsdk.config import API
from boxsdk.object.group import Group
from boxsdk.object.item import Item
from boxsdk.object.user import User
from boxsdk.pagination.limit_offset_based_object_collection import LimitOffsetBasedObjectCollection
from boxsdk.pagination.marker_based_object_collection import MarkerBasedObjectCollection
from boxsdk.util.api_call_decorator import api_call
from boxsdk.util.text_enum import TextEnum


class FolderSyncState(TextEnum):
    """An enum of all possible values of a folder's ``sync_state`` attribute.

    The value of the ``sync_state`` attribute determines whether the folder
    will be synced by sync clients.
    """
    IS_SYNCED = 'synced'
    NOT_SYNCED = 'not_synced'
    PARTIALLY_SYNCED = 'partially_synced'


class _CollaborationType(TextEnum):
    """The type of a collaboration"""
    USER = 'user'
    GROUP = 'group'


class _Collaborator(object):
    """This helper class represents a collaborator on Box. A Collaborator can be a User, Group, or an email address"""
    def __init__(self, collaborator):
        if isinstance(collaborator, User):
            self._setup(user=collaborator)
        elif isinstance(collaborator, Group):
            self._setup(group=collaborator)
        elif isinstance(collaborator, text_type):
            self._setup(email_address=collaborator)
        else:
            raise TypeError('Collaborator must be User, Group, or unicode string')

    def _setup(self, user=None, group=None, email_address=None):
        """
        :param user:
            The Box user if applicable
        :type user:
            :class:`User`
        :param group:
            The Box group if applicable
        :type group:
            :class:`Group`
        :param email_address:
            The email address of the user if not a user of Box
        :type email_address:
            `unicode`
        """
        self._type = _CollaborationType.GROUP if group else _CollaborationType.USER
        id_object = user or group
        if id_object:
            self._key = 'id'
            self._identifier = id_object.object_id
        else:
            self._key = 'login'
            self._identifier = email_address

    @property
    def access(self):
        """Return a tuple for how to access collaborator

        The first element is the key for access, the second is the value
        :rtype:
            `tuple` of `unicode`, `unicode`
        """
        return self._key, self._identifier

    @property
    def type(self):
        """Return the type of collaborator (user or group)
        :rtype:
            `unicode`
        """
        return self._type


class Folder(Item):
    """Box API endpoint for interacting with folders."""

    _item_type = 'folder'

    def preflight_check(self, size, name):
        """
        Make an API call to check if a new file with given name and size can be uploaded to this folder.

        :param size:
            The size of the file in bytes. Specify 0 for unknown file-sizes.
        :type size:
            `int`
        :param name:
            The name of the file to be uploaded.
        :type name:
            `unicode`
        :raises:
            :class:`BoxAPIException` when preflight check fails.
        """
        self._preflight_check(
            size=size,
            name=name,
            parent_id=self._object_id,
        )

    def _get_accelerator_upload_url_fow_new_uploads(self):
        """
        Get Accelerator upload url for uploading new files.

        :return:
            The Accelerator upload url or None if cannot get one
        :rtype:
            `unicode` or None
        """
        return self._get_accelerator_upload_url()

    @api_call
    def get_items(self, limit, offset=0, fields=None):
        """Get the items in a folder.

        :param limit:
            The maximum number of items to return.
        :type limit:
            `int`
        :param offset:
            The index at which to start returning items.
        :type offset:
            `int`
        :param fields:
            List of fields to request.
        :type fields:
            `Iterable` of `unicode`
        :returns:
            A list of items in the folder.
        :rtype:
            `list` of :class:`Item`
        """
        url = self.get_url('items')
        params = {
            'limit': limit,
            'offset': offset,
        }
        if fields:
            params['fields'] = ','.join(fields)
        box_response = self._session.get(url, params=params)
        response = box_response.json()
        return [self.translator.translate(item['type'])(self._session, item['id'], item) for item in response['entries']]

    @api_call
    def get_items_limit_offset(self, limit=None, offset=0, fields=None):
        """
        Get the items in a folder using limit-offset paging.

        :param limit:
            The maximum number of items to return per page. If not specified, then will use the server-side default.
        :type limit:
            `int` or None
        :param offset:
            The index at which to start returning items.
        :type offset:
            `int`
        :param fields:
            List of fields to request.
        :type fields:
            `Iterable` of `unicode`
        :returns:
            An iterator of the items in the folder.
        :rtype:
            :class:`BoxObjectCollection`
        """
        return LimitOffsetBasedObjectCollection(
            self.session,
            self.get_url('items'),
            limit=limit,
            fields=fields,
            offset=offset,
            return_full_pages=False,
        )

    @api_call
    def get_items_marker(self, limit=None, marker=None, fields=None):
        """
        Get the items in a folder using marker-based paging.

        :param limit:
            The maximum number of items to return per page. If not specified, then will use the server-side default.
        :type limit:
            `int` or None
        :param marker:
            The offset index to start paging from.
        :type marker:
            `str` or None
        :param fields:
            List of fields to request.
        :type fields:
            `Iterable` of `unicode`
        :returns:
            An iterator of the items in the folder.
        :rtype:
            :class:`BoxObjectCollection`
        """
        return MarkerBasedObjectCollection(
            self.session,
            self.get_url('items'),
            limit=limit,
            fields=fields,
            marker=marker,
            return_full_pages=False,
            supports_limit_offset_paging=True,
        )

    @api_call
    def upload_stream(
            self,
            file_stream,
            file_name,
            preflight_check=False,
            preflight_expected_size=0,
            upload_using_accelerator=False,
    ):
        """
        Upload a file to the folder.
        The contents are taken from the given file stream, and it will have the given name.

        :param file_stream:
            The file-like object containing the bytes
        :type file_stream:
            `file`
        :param file_name:
            The name to give the file on Box.
        :type file_name:
            `unicode`
        :param preflight_check:
            If specified, preflight check will be performed before actually uploading the file.
        :type preflight_check:
            `bool`
        :param preflight_expected_size:
            The size of the file to be uploaded in bytes, which is used for preflight check. The default value is '0',
            which means the file size is unknown.
        :type preflight_expected_size:
            `int`
        :param upload_using_accelerator:
            If specified, the upload will try to use Box Accelerator to speed up the uploads for big files.
            It will make an extra API call before the actual upload to get the Accelerator upload url, and then make
            a POST request to that url instead of the default Box upload url. It falls back to normal upload endpoint,
            if cannot get the Accelerator upload url.

            Please notice that this is a premium feature, which might not be available to your app.
        :type upload_using_accelerator:
            `bool`
        :returns:
            The newly uploaded file.
        :rtype:
            :class:`File`
        """
        if preflight_check:
            self.preflight_check(size=preflight_expected_size, name=file_name)

        url = '{0}/files/content'.format(API.UPLOAD_URL)
        if upload_using_accelerator:
            accelerator_upload_url = self._get_accelerator_upload_url_fow_new_uploads()
            if accelerator_upload_url:
                url = accelerator_upload_url

        data = {'attributes': json.dumps({
            'name': file_name,
            'parent': {'id': self._object_id},
        })}
        files = {
            'file': ('unused', file_stream),
        }
        file_response = self._session.post(url, data=data, files=files, expect_json_response=False).json()
        if 'entries' in file_response:
            file_response = file_response['entries'][0]
        file_id = file_response['id']
        return self.translator.translate(file_response['type'])(
            session=self._session,
            object_id=file_id,
            response_object=file_response,
        )

    @api_call
    def upload(
            self,
            file_path=None,
            file_name=None,
            preflight_check=False,
            preflight_expected_size=0,
            upload_using_accelerator=False,
    ):
        """
        Upload a file to the folder.
        The contents are taken from the given file path, and it will have the given name.
        If file_name is not specified, the uploaded file will take its name from file_path.

        :param file_path:
            The file path of the file to upload to Box.
        :type file_path:
            `unicode`
        :param file_name:
            The name to give the file on Box. If None, then use the leaf name of file_path
        :type file_name:
            `unicode`
        :param preflight_check:
            If specified, preflight check will be performed before actually uploading the file.
        :type preflight_check:
            `bool`
        :param preflight_expected_size:
            The size of the file to be uploaded in bytes, which is used for preflight check. The default value is '0',
            which means the file size is unknown.
        :type preflight_expected_size:
            `int`
        :param upload_using_accelerator:
            If specified, the upload will try to use Box Accelerator to speed up the uploads for big files.
            It will make an extra API call before the actual upload to get the Accelerator upload url, and then make
            a POST request to that url instead of the default Box upload url. It falls back to normal upload endpoint,
            if cannot get the Accelerator upload url.

            Please notice that this is a premium feature, which might not be available to your app.
        :type upload_using_accelerator:
            `bool`
        :returns:
            The newly uploaded file.
        :rtype:
            :class:`File`
        """
        if file_name is None:
            file_name = os.path.basename(file_path)
        with open(file_path, 'rb') as file_stream:
            return self.upload_stream(
                file_stream,
                file_name,
                preflight_check,
                preflight_expected_size=preflight_expected_size,
                upload_using_accelerator=upload_using_accelerator,
            )

    @api_call
    def create_subfolder(self, name):
        """
        Create a subfolder with the given name in the folder.

        :param name:
            The name of the new folder
        :type name:
            `unicode`
        """
        url = self.get_type_url()
        data = {
            'name': name,
            'parent': {
                'id': self._object_id,
            }
        }
        box_response = self._session.post(url, data=json.dumps(data))
        response = box_response.json()
        return self.__class__(
            session=self._session,
            object_id=response['id'],
            response_object=response,
        )

    @api_call
    def update_sync_state(self, sync_state):
        """Update the ``sync_state`` attribute of this folder.

        Change whether this folder will be synced by sync clients.

        :param sync_state:
            The desired sync state of this folder.
            Must be a member of the `FolderSyncState` enum.
        :type sync_state:
            :class:`FolderSyncState`
        :return:
            A new :class:`Folder` instance with updated information reflecting the new sync state.
        :rtype:
            :class:`Folder`
        """
        data = {
            'sync_state': sync_state,
        }
        return self.update_info(data=data)

    @api_call
    def add_collaborator(self, collaborator, role, notify=False, can_view_path=False):
        """Add a collaborator to the folder

        :param collaborator:
            collaborator to add. May be a User, Group, or email address (unicode string)
        :type collaborator:
            :class:`User` or :class:`Group` or `unicode`
        :param role:
            The collaboration role
        :type role:
            :class:`CollaboratorRole`
        :param notify:
            Whether to send a notification email to the collaborator
        :type notify:
            `bool`
        :param can_view_path:
            Whether view path collaboration feature is enabled or not. Note - only
            folder owners can create collaborations with can_view_path.
        :type can_view_path:
            `bool`
        :return:
            The new collaboration
        :rtype:
            :class:`Collaboration`
        """
        collaborator_helper = _Collaborator(collaborator)
        url = API.BASE_API_URL + '/collaborations'
        item = {'id': self._object_id, 'type': 'folder'}
        access_key, access_value = collaborator_helper.access
        accessible_by = {
            access_key: access_value,
            'type': collaborator_helper.type,
        }
        body_params = {
            'item': item,
            'accessible_by': accessible_by,
            'role': role,
        }
        if can_view_path:
            body_params['can_view_path'] = True
        data = json.dumps(body_params)
        params = {'notify': notify}
        box_response = self._session.post(url, expect_json_response=True, data=data, params=params)
        collaboration_response = box_response.json()
        collab_id = collaboration_response['id']
        return self.translator.translate(collaboration_response['type'])(
            session=self._session,
            object_id=collab_id,
            response_object=collaboration_response,
        )

    @api_call
    def delete(self, recursive=True, etag=None):
        """Base class override. Delete the folder.

        :param recursive:
            Whether or not the folder should be deleted if it isn't empty.
        :type recursive:
            `bool`
        :param etag:
            If specified, instruct the Box API to delete the folder only if the current version's etag matches.
        :type etag:
            `unicode` or None
        :returns:
            Whether or not the update was successful.
        :rtype:
            `bool`
        :raises: :class:`BoxAPIException` if the specified etag doesn't match the latest version of the folder.
        """
        # pylint:disable=arguments-differ
        return super(Folder, self).delete({'recursive': recursive}, etag)



# coding: utf-8

from __future__ import unicode_literals, absolute_import

from .client import Client
from .developer_token_client import DeveloperTokenClient
from .development_client import DevelopmentClient
from .logging_client import LoggingClient



# coding: utf-8

from __future__ import unicode_literals, absolute_import

from collections import Mapping
from abc import ABCMeta

from .base_api_json_object import BaseAPIJSONObject, BaseAPIJSONObjectMeta
from ..util.compat import with_metaclass


class APIJSONObjectMeta(BaseAPIJSONObjectMeta, ABCMeta):
    """
    Avoid conflicting metaclass definitions for APIJSONObject.
    http://code.activestate.com/recipes/204197-solving-the-metaclass-conflict/
    """
    pass


class APIJSONObject(with_metaclass(APIJSONObjectMeta, BaseAPIJSONObject, Mapping)):
    """Class representing objects that are not part of the REST API."""

    def __len__(self):
        return len(self._response_object)



# coding: utf-8

from __future__ import unicode_literals
from .oauth2 import OAuth2


class RemoteOAuth2Mixin(OAuth2):
    """
    Box SDK OAuth2 mixin.
    Allows for storing auth tokens remotely.

    """
    def __init__(self, retrieve_access_token=None, *args, **kwargs):
        """
        :param retrieve_access_token:
            Callback to exchange an existing access token for a new one.
        :type retrieve_access_token:
            `callable` of `unicode` => `unicode`
        """
        self._retrieve_access_token = retrieve_access_token
        super(RemoteOAuth2Mixin, self).__init__(*args, **kwargs)

    def _refresh(self, access_token):
        """
        Base class override. Ask the remote host for a new token.
        """
        self._access_token = self._retrieve_access_token(access_token)
        return self._access_token, None


class RemoteOAuth2(RemoteOAuth2Mixin):
    """
    Box SDK OAuth2 subclass.
    Allows for storing auth tokens remotely. The retrieve_access_token callback should
    return an access token, presumably acquired from a remote server on which your auth credentials are available.
    """
    pass



# coding: utf-8

from __future__ import unicode_literals, absolute_import

from .base_object import BaseObject


class GroupMembership(BaseObject):
    """Represents a Box group_membership, which relates a user & group under a specific role."""

    _item_type = 'group_membership'

    def __init__(self, session, object_id, response_object=None, user=None, group=None):
        """
        :param session:
            The Box session used to make requests.
        :type session:
            :class:`BoxSession`
        :param object_id:
            The Box ID for the object.
        :type object_id:
            `unicode`
        :param response_object:
            The Box API response representing the object.
        :type response_object:
            :class:`BoxResponse`
        :param user:
            The User object, if any, associated with this group_membership instance.
        :type user:
            :class:`User`
        :param group:
            The Group object, if any, associated with this group_membership instance.
        :type group:
            :class:`Group`
        """
        user, group = self._init_user_and_group_instances(session, response_object, user, group)

        super(GroupMembership, self).__init__(session, object_id, response_object)

        self.user = user
        self.group = group

    @staticmethod
    def _init_user_and_group_instances(session, response_object, user, group):
        """
        Initialize User & Group instances based on the passed in parameters. This allows the GroupMembership
        instance to maintain a 'has-a' relationship with a corresponding User and Group instance.

        :param session:
            The Box session used to make requests.
        :type session:
            :class:`BoxSession`
        :param response_object:
            The Box API response representing the object.
        :type response_object:
            :class:`BoxResponse`
        :param user:
            The User object, if any, to associate with this group_membership instance.
        :type user:
            :class:`User`
        :param group:
            The Group object, if any, to associate with this group_membership instance.
        :type group:
            :class:`Group`
        """
        if not response_object:
            return user, group

        user_info = response_object.get('user')
        group_info = response_object.get('group')

        user = user or session.translator.translate(user_info['type'])(session, user_info['id'], user_info)
        group = group or session.translator.translate(group_info['type'])(session, group_info['id'], group_info)

        return user, group

    def clone(self, session=None):
        """Base class override."""
        return self.__class__(
            session or self._session,
            self._object_id,
            self._response_object,
            self.user,
            self.group,
        )



# coding: utf-8

from __future__ import unicode_literals

from itertools import chain, islice, repeat, count
import json
from operator import sub

from mock import Mock
import pytest
from six.moves import map  # pylint:disable=redefined-builtin,import-error

from boxsdk.network.default_network import DefaultNetworkResponse
from boxsdk.object.group_membership import GroupMembership
from boxsdk.config import API
from boxsdk.session.box_session import BoxResponse


@pytest.fixture(scope='module')
def delete_group_response():
    # pylint:disable=redefined-outer-name
    mock_network_response = Mock(DefaultNetworkResponse)
    mock_network_response.ok = True
    return mock_network_response


def test_delete_group_return_the_correct_response(
        mock_group,
        mock_box_session,
        delete_group_response,
):
    # pylint:disable=redefined-outer-name
    mock_box_session.delete.return_value = delete_group_response
    response = mock_group.delete()

    # pylint:disable=protected-access
    expected_url = mock_group.get_url()
    # pylint:enable=protected-access
    mock_box_session.delete.assert_called_once_with(expected_url, params={}, expect_json_response=False, headers=None)

    assert response is True


@pytest.mark.parametrize('role', ['member', 'admin'])
def test_add_member(test_group, mock_box_session, mock_add_member_response, mock_user, role):
    expected_url = '{0}/group_memberships'.format(API.BASE_API_URL)
    mock_box_session.post.return_value = mock_add_member_response
    new_group_membership = test_group.add_member(mock_user, role)
    data = json.dumps({
        'user': {'id': mock_user.object_id},
        'group': {'id': test_group.object_id},
        'role': role,
    })
    mock_box_session.post.assert_called_once_with(expected_url, data=data)
    assert isinstance(new_group_membership, GroupMembership)


@pytest.fixture()
def mock_membership_dict_stream():
    def gen_data(some_id):
        return {
            'type': 'group_membership',
            'id': "membership_id_{0}".format(some_id),
            'role': 'member',
            'user': {'type': 'user', 'id': "user_id_{0}".format(some_id)},
            'group': {'type': 'group', 'id': "group_id_{0}".format(some_id)},
        }

    return map(gen_data, count())


@pytest.fixture()
def mock_membership_responses(mock_membership_dict_stream):
    """
    Returns a generator method that takes params: total, page_size.
    The generator generates a sequence of 'group membership' mock_box_responses each containing page_size
    items, until 'total' entries have been returned
    """
    # pylint:disable=redefined-outer-name
    def number_entries_per_response(total, page_size, hidden_in_batch):
        if not hidden_in_batch:
            hidden_in_batch = repeat(0)
        quotient, remainder = divmod(total, page_size)
        max_items_in_batch = chain(repeat(page_size, quotient), (remainder,))
        return map(sub, max_items_in_batch, hidden_in_batch)

    def take(iterable, number):
        return list(islice(iterable, number))

    def membership_responses(total, page_size, hidden_in_batch=None):
        offset = 0
        for number_entries in number_entries_per_response(total, page_size, hidden_in_batch):
            entries = take(mock_membership_dict_stream, number_entries)

            mock_box_response = Mock(BoxResponse)
            mock_network_response = Mock(DefaultNetworkResponse)
            mock_box_response.network_response = mock_network_response
            mock_box_response.json.return_value = {
                'entries': entries,
                'total_count': total,
                'offset': offset,
                'limit': page_size,
            }
            offset += number_entries
            mock_box_response.status_code = 200
            mock_box_response.ok = True
            yield mock_box_response

    return membership_responses


@pytest.mark.parametrize('total, page_size', [
    (0, 6),
    (5, 6),
    (6, 6),
    (5, 4),
    (9, 4),
])
def test_membership(test_group, mock_box_session, mock_membership_responses, total, page_size):
    # pylint:disable=redefined-outer-name
    # Each call the 'get' (the GET next page call) will return the next response
    mock_box_session.get.side_effect = mock_membership_responses(total, page_size)

    # Get all the members
    all_members = list(test_group.membership(0, page_size))

    # Assert we got the expected number of membership instances
    assert len(all_members) == total
    assert all(isinstance(m, GroupMembership) for m in all_members)


@pytest.mark.parametrize('hidden_in_batch', [
    (1, 0, 0),
    (0, 0, 1),
    (10, 10, 9),
    (10, 10, 10),
])
def test_membership_with_hidden_results(test_group, mock_box_session, mock_membership_responses, hidden_in_batch):
    """
    This test verifies that the SDK properly deals with missing (aka hidden) data potentially present in a paged
    API. The API might indicate that the total_size is X, but in actuality the pages API could return less than X
    because the auth'd user might not have access to all X of the resources.
    """
    # pylint:disable=redefined-outer-name
    total = 30
    page_size = 10

    total_hidden = sum(hidden_in_batch)

    # Each call the 'get' (the GET next page call) will return the next response
    mock_box_session.get.side_effect = mock_membership_responses(total, page_size, hidden_in_batch=hidden_in_batch)

    # Get all the members
    all_members = list(test_group.membership(0, page_size))

    # Assert we got the expected number of membership instances
    assert len(all_members) == total - total_hidden
    assert all(isinstance(m, GroupMembership) for m in all_members)


def test_membership_with_page_info(test_group, mock_box_session, mock_membership_responses):
    """
    Verify that the paging info returned by the membership call when include_page_info=True
    is correct, thus allowing a client complete knowledge of when another API call
    is going to be triggered.
    """
    # pylint:disable=redefined-outer-name
    total = 9
    page_size = 3
    hidden_in_batch = 0, 2, 1

    # Each call to 'get' (the GET next page call) will return the next response
    mock_box_session.get.side_effect = mock_membership_responses(total, page_size, hidden_in_batch=hidden_in_batch)

    # Initialize the generator of all the membership
    group_generator = test_group.membership(0, page_size, include_page_info=True)

    # manually get all the items, verifying that the page-info data is correct.
    _, page_size, index = next(group_generator)
    assert page_size == 3 and index == 0
    _, page_size, index = next(group_generator)
    assert page_size == 3 and index == 1
    _, page_size, index = next(group_generator)
    assert page_size == 3 and index == 2

    # This next call will trigger a new GET request, returning the 2nd page
    _, page_size, index = next(group_generator)
    assert page_size == 1 and index == 0

    # This next call will trigger a new GET request, returning the 3rd page
    _, page_size, index = next(group_generator)
    assert page_size == 2 and index == 0
    _, page_size, index = next(group_generator)
    assert page_size == 2 and index == 1

    with pytest.raises(StopIteration):
        next(group_generator)



# coding: utf-8

from __future__ import unicode_literals, absolute_import
import json

from ..auth.oauth2 import TokenResponse
from ..config import API
from ..session.box_session import BoxSession
from ..network.default_network import DefaultNetwork
from ..object.cloneable import Cloneable
from ..util.api_call_decorator import api_call
from ..object.search import Search
from ..object.events import Events
from ..pagination.marker_based_object_collection import MarkerBasedObjectCollection
from ..util.shared_link import get_shared_link_header


class Client(Cloneable):

    def __init__(
            self,
            oauth=None,
            network_layer=None,
            session=None,
    ):
        """
        :param oauth:
            OAuth2 object used by the session to authorize requests.
        :type oauth:
            :class:`OAuth2`
        :param network_layer:
            The Network layer to use. If none is provided then an instance of :class:`DefaultNetwork` will be used.
        :type network_layer:
            :class:`Network`
        :param session:
            The session object to use. If None is provided then an instance of :class:`BoxSession` will be used.
        :type session:
            :class:`BoxSession`
        """
        super(Client, self).__init__()
        self._oauth = oauth
        self._session = session or BoxSession(oauth=oauth, network_layer=(network_layer or DefaultNetwork()))

    @property
    def auth(self):
        """
        Get the :class:`OAuth2` instance the client is using for auth to Box.

        :rtype:
            :class:`OAuth2`
        """
        return self._oauth

    @property
    def session(self):
        """
        Get the :class:`BoxSession` instance the client is using.

        :rtype:
            :class:`BoxSession`
        """
        return self._session

    @property
    def translator(self):
        """The translator used for translating Box API JSON responses into `BaseAPIJSONObject` smart objects.

        :rtype:   :class:`Translator`
        """
        return self._session.translator

    def folder(self, folder_id):
        """
        Initialize a :class:`Folder` object, whose box id is folder_id.

        :param folder_id:
            The box id of the :class:`Folder` object. Can use '0' to get the root folder on Box.
        :type folder_id:
            `unicode`
        :return:
            A :class:`Folder` object with the given folder id.
        :rtype:
            :class:`Folder`
        """
        return self.translator.translate('folder')(session=self._session, object_id=folder_id)

    def file(self, file_id):
        """
        Initialize a :class:`File` object, whose box id is file_id.

        :param file_id:
            The box id of the :class:`File` object.
        :type file_id:
            `unicode`
        :return:
            A :class:`File` object with the given file id.
        :rtype:
            :class:`File`
        """
        return self.translator.translate('file')(session=self._session, object_id=file_id)

    def user(self, user_id='me'):
        """
        Initialize a :class:`User` object, whose box id is user_id.

        :param user_id:
            The user id of the :class:`User` object. Can use 'me' to get the User for the current/authenticated user.
        :type user_id:
            `unicode`
        :return:
            A :class:`User` object with the given id.
        :rtype:
            :class:`User`
        """
        return self.translator.translate('user')(session=self._session, object_id=user_id)

    def group(self, group_id):
        """
        Initialize a :class:`Group` object, whose box id is group_id.

        :param group_id:
            The box id of the :class:`Group` object.
        :type group_id:
            `unicode`
        :return:
            A :class:`Group` object with the given group id.
        :rtype:
            :class:`Group`
        """
        return self.translator.translate('group')(session=self._session, object_id=group_id)

    def collaboration(self, collab_id):
        """
        Initialize a :class:`Collaboration` object, whose box id is collab_id.

        :param collab_id:
            The box id of the :class:`Collaboration` object.
        :type collab_id:
            `unicode`
        :return:
            A :class:`Collaboration` object with the given group id.
        :rtype:
            :class:`Collaboration`
        """
        return self.translator.translate('collaboration')(session=self._session, object_id=collab_id)

    @api_call
    def users(self, limit=None, offset=0, filter_term=None):
        """
        Get a list of all users for the Enterprise along with their user_id, public_name, and login.

        :param limit:
            The maximum number of users to return. If not specified, the Box API will determine an appropriate limit.
        :type limit:
            `int` or None
        :param offset:
            The user index at which to start the response.
        :type offset:
            `int`
        :param filter_term:
            Filters the results to only users starting with the filter_term in either the name or the login.
        :type filter_term:
            `unicode` or None
        :return:
            The list of all users in the enterprise.
        :rtype:
            `list` of :class:`User`
        """
        url = '{0}/users'.format(API.BASE_API_URL)
        params = dict(offset=offset)
        if limit is not None:
            params['limit'] = limit
        if filter_term is not None:
            params['filter_term'] = filter_term
        box_response = self._session.get(url, params=params)
        response = box_response.json()
        user_class = self.translator.translate('user')
        return [user_class(
            session=self._session,
            object_id=item['id'],
            response_object=item,
        ) for item in response['entries']]

    @api_call
    def search(
            self,
            query,
            limit,
            offset,
            ancestor_folders=None,
            file_extensions=None,
            metadata_filters=None,
            result_type=None,
            content_types=None
    ):
        """
        Search Box for items matching the given query.

        :param query:
            The string to search for.
        :type query:
            `unicode`
        :param limit:
            The maximum number of items to return.
        :type limit:
            `int`
        :param offset:
            The search result at which to start the response.
        :type offset:
            `int`
        :param ancestor_folders:
            Folder ids to limit the search to.
        :type ancestor_folders:
            `iterable` of :class:`Folder`
        :param file_extensions:
            File extensions to limit the search to.
        :type file_extensions:
            `iterable` of `unicode`
        :param metadata_filters:
            Filters used for metadata search
        :type metadata_filters:
            :class:`MetadataSearchFilters`
        :param result_type:
            Which type of result you want. Can be file or folder.
        :type result_type:
            `unicode`
        :param content_types:
            Which content types to search. Valid types include name, description, file_content, comments, and tags.
        :type content_types:
            `Iterable` of `unicode`
        :return:
            A list of items that match the search query.
        :rtype:
            `list` of :class:`Item`
        """
        return Search(self._session).search(
            query=query,
            limit=limit,
            offset=offset,
            ancestor_folders=ancestor_folders,
            file_extensions=file_extensions,
            metadata_filters=metadata_filters,
            result_type=result_type,
            content_types=content_types,
        )

    def events(self):
        """
        Get an events object that can get the latest events from Box or set up a long polling event subscription.
        """
        return Events(self._session)

    def group_membership(self, group_membership_id):
        """
        Initialize a :class:`GroupMembership` object, whose box id is group_membership_id.

        :param group_membership_id:
            The box id of the :class:`GroupMembership` object.
        :type group_membership_id:
            `unicode`
        :return:
            A :class:`GroupMembership` object with the given membership id.
        :rtype:
            :class:`GroupMembership`
        """
        return self.translator.translate('group_membership')(
            session=self._session,
            object_id=group_membership_id,
        )

    @api_call
    def groups(self):
        """
        Get a list of all groups for the current user.

        :return:
            The list of all groups.
        :rtype:
            `list` of :class:`Group`
        """
        url = '{0}/groups'.format(API.BASE_API_URL)
        box_response = self._session.get(url)
        response = box_response.json()
        group_class = self.translator.translate('group')
        return [group_class(
            session=self._session,
            object_id=item['id'],
            response_object=item,
        ) for item in response['entries']]

    @api_call
    def create_group(self, name):
        """
        Create a group with the given name.

        :param name:
            The name of the group.
        :type name:
            `unicode`
        :return:
            The newly created Group.
        :rtype:
            :class:`Group`
        :raises:
            :class:`BoxAPIException` if current user doesn't have permissions to create a group.
        """
        url = '{0}/groups'.format(API.BASE_API_URL)
        body_attributes = {
            'name': name,
        }
        box_response = self._session.post(url, data=json.dumps(body_attributes))
        response = box_response.json()
        return self.translator.translate('group')(
            session=self._session,
            object_id=response['id'],
            response_object=response,
        )

    @api_call
    def get_recent_items(self, limit=None, marker=None, fields=None, **collection_kwargs):
        """
        Get the user's recently accessed items.

        :param: limit
            The maximum number of items to return. If limit is set to None, then the default
            limit (returned by Box in the response) is used. See https://developer.box.com/reference#get-recent-items
            for default.
        :type: limit
            `int` or None
        :param marker:
            The index at which to start returning items.
        :type marker:
            `str` or None
        :param fields:
            List of fields to request on the file or folder which the `RecentItem` references.
        :type fields:
            `Iterable` of `unicode`
        :param **collection_kwargs:
            Keyword arguments passed to `MarkerBasedObjectCollection`.
        :type **collection_args:
            `dict`
        :returns:
            An iterator on the user's recent items
        :rtype:
            :class:`MarkerBasedObjectCollection`
        """
        return MarkerBasedObjectCollection(
            self.session,
            self.get_url('recent_items'),
            limit=limit,
            fields=fields,
            marker=marker,
            **collection_kwargs
        )

    @api_call
    def get_shared_item(self, shared_link, password=None):
        """
        Get information about a Box shared link. https://box-content.readme.io/reference#get-a-shared-item

        :param shared_link:
            The shared link.
        :type shared_link:
            `unicode`
        :param password:
            The password for the shared link.
        :type password:
            `unicode`
        :return:
            The item referred to by the shared link.
        :rtype:
            :class:`Item`
        :raises:
            :class:`BoxAPIException` if current user doesn't have permissions to view the shared link.
        """
        response = self.make_request(
            'GET',
            '{0}/shared_items'.format(API.BASE_API_URL),
            headers=get_shared_link_header(shared_link, password),
        ).json()
        return self.translator.translate(response['type'])(
            session=self._session.with_shared_link(shared_link, password),
            object_id=response['id'],
            response_object=response,
        )

    @api_call
    def make_request(self, method, url, **kwargs):
        """
        Make an authenticated request to the Box API.

        :param method:
            The HTTP verb to use for the request.
        :type method:
            `unicode`
        :param url:
            The URL for the request.
        :type url:
            `unicode`
        :return:
            The network response for the given request.
        :rtype:
            :class:`BoxResponse`
        :raises:
            :class:`BoxAPIException`
        """
        return self._session.request(method, url, **kwargs)

    @api_call
    def create_user(self, name, login=None, **user_attributes):
        """
        Create a new user. Can only be used if the current user is an enterprise admin, or the current authorization
        scope is a Box developer edition instance.

        :param name:
            The user's display name.
        :type name:
            `unicode`
        :param login:
            The user's email address. Required for an enterprise user, but None for an app user.
        :type login:
            `unicode` or None
        :param user_attributes:
            Additional attributes for the user. See the documentation at
            https://box-content.readme.io/#create-an-enterprise-user for enterprise users
            or https://box-content.readme.io/docs/app-users for app users.
        """
        url = '{0}/users'.format(API.BASE_API_URL)
        user_attributes['name'] = name
        if login is not None:
            user_attributes['login'] = login
        else:
            user_attributes['is_platform_access_only'] = True
        box_response = self._session.post(url, data=json.dumps(user_attributes))
        response = box_response.json()
        return self.translator.translate('user')(
            session=self._session,
            object_id=response['id'],
            response_object=response,
        )

    def downscope_token(self, scopes, item=None, additional_data=None):
        """
        Generate a downscoped token for the provided file or folder with the provided scopes.

        :param scope:
            The scope(s) to apply to the resulting token.
        :type scopes:
            `Iterable` of :class:`TokenScope`
        :param item:
            (Optional) The file or folder to get a downscoped token for. If None, the resulting token will
            not be scoped down to just a single item.
        :type item:
            :class:`Item`
        :param additional_data:
            (Optional) Key value pairs which can be used to add/update the default data values in the request.
        :type additional_data:
            `dict`
        :return:
            The response for the downscope token request.
        :rtype:
            :class:`TokenResponse`
        """
        url = '{base_auth_url}/token'.format(base_auth_url=API.OAUTH2_API_URL)
        data = {
            'subject_token': self.auth.access_token,
            'subject_token_type': 'urn:ietf:params:oauth:token-type:access_token',
            'scope': ' '.join(scopes),
            'grant_type': 'urn:ietf:params:oauth:grant-type:token-exchange',
        }
        if item:
            data['resource'] = item.get_url()
        if additional_data:
            data.update(additional_data)

        box_response = self._session.post(url, data=data)
        return TokenResponse(box_response.json())

    def clone(self, session=None):
        """Base class override."""
        return self.__class__(oauth=self._oauth, session=(session or self._session))

    def get_url(self, endpoint, *args):
        """
        Return the URL for the given Box API endpoint.

        :param endpoint:
            The name of the endpoint.
        :type endpoint:
            `url`
        :param args:
            Additional parts of the endpoint URL.
        :type args:
            `Iterable`
        :rtype:
            `unicode`
        """
        # pylint:disable=no-self-use
        return self._session.get_url(endpoint, *args)



# coding: utf-8

from __future__ import unicode_literals

from .box_object_collection import BoxObjectCollection


class MarkerBasedObjectCollection(BoxObjectCollection):
    """
    An iterator of Box objects (BaseObjects) that were retrieved from a Box API endpoint that supports
    marker type of pagination.

    See https://developer.box.com/reference#pagination for more details.
    """

    def __init__(
            self,
            session,
            url,
            limit=None,
            fields=None,
            additional_params=None,
            return_full_pages=False,
            marker=None,
            supports_limit_offset_paging=False,
    ):
        """
        :param marker:
            The offset index to start paging from.
        :type marker:
            `str` or None
        :param supports_limit_offset_paging:
            Does this particular endpoint also support limit-offset paging? This information is needed, as
            the endpoints that support both require an special extra request parameter.
        :type supports_limit_offset_paging:
            `bool`
        """
        super(MarkerBasedObjectCollection, self).__init__(
            session,
            url,
            limit=limit,
            fields=fields,
            additional_params=additional_params,
            return_full_pages=return_full_pages,
        )
        self._marker = marker
        self._supports_limit_offset_paging = supports_limit_offset_paging

    def _update_pointer_to_next_page(self, response_object):
        """Baseclass override."""
        self._marker = self._get_next_marker_from_response_object(response_object)

    def _has_more_pages(self, response_object):
        """Baseclass override."""
        return bool(self._get_next_marker_from_response_object(response_object))

    @staticmethod
    def _get_next_marker_from_response_object(response_object):
        """Get the marker that should be used to retrieve the next page.

        When we've just retrieved the last page, the API is inconsistent about
        what it returns. Some endpoints return "next_marker":"", some return
        "next_marker":null, some don't give any "next_marker" value. In all of
        these cases, this method will return `None`.

        Otherwise, this method returns the string value of the "next_marker"
        field.

        :rtype:
            `unicode` or `None`
        """
        return response_object.get('next_marker') or None

    def _next_page_pointer_params(self):
        """Baseclass override."""
        pointer_params = {}
        # For transitioning endpoints that support both marker and limit-offset paging, we must specify an
        # additional 'useMarker' parameter to the Box API.
        if self._supports_limit_offset_paging:
            pointer_params['useMarker'] = True
        if self._marker is not None:
            pointer_params['marker'] = self._marker
        return pointer_params

    def next_pointer(self):
        """Baseclass override."""
        return self._marker



# coding: utf-8

from __future__ import unicode_literals, absolute_import

from itertools import chain
import json

from mock import Mock
import pytest
from requests.exceptions import Timeout
from six.moves import map   # pylint:disable=redefined-builtin
from six.moves.urllib.parse import urlencode, urlunsplit  # pylint:disable=import-error,no-name-in-module

from boxsdk.network.default_network import DefaultNetworkResponse
from boxsdk.object.events import Events, EventsStreamType, UserEventsStreamType
from boxsdk.object.event import Event
from boxsdk.session.box_session import BoxResponse
from boxsdk.util.ordered_dict import OrderedDict


@pytest.fixture()
def test_events(mock_box_session):
    return Events(mock_box_session)


@pytest.fixture()
def final_stream_position():
    return 1348790499820


@pytest.fixture()
def initial_stream_position():
    return 1348790499819


# pylint:disable=no-member
# pylint isn't currently smart enough to recognize the class member that was
# added by the metaclass, when the metaclass was added by @add_metaclass() /
# with_metaclass().
STREAM_TYPES_AS_ENUM_INSTANCES = list(EventsStreamType.__members__.values())
# pylint:enable=no-member
STREAM_TYPES_AS_STRINGS = list(map(str, STREAM_TYPES_AS_ENUM_INSTANCES))


def test_events_stream_type_extended_enum_class_has_expected_members():
    assert len(STREAM_TYPES_AS_ENUM_INSTANCES) >= 4
    assert len(STREAM_TYPES_AS_STRINGS) >= 4
    assert 'all' in STREAM_TYPES_AS_STRINGS
    assert 'changes' in STREAM_TYPES_AS_STRINGS
    assert 'sync' in STREAM_TYPES_AS_STRINGS
    assert 'admin_logs' in STREAM_TYPES_AS_STRINGS


@pytest.fixture(
    scope='session',
    params=list(chain(
        [None],   # Default behavior of not passing any stream_type
        STREAM_TYPES_AS_ENUM_INSTANCES,   # Passing an enum instance
        STREAM_TYPES_AS_STRINGS,  # Passing an enum value

        # For forwards compatibility, make sure that it works to pass a string
        # value that is not a member of the enum.
        ['future_stream_type'],
    )),
)
def stream_type_param(request):
    """The value to pass as an Event method's stream_type parameter.

    :return:
        The parameter value, or `None` if no value should be passed.
    :rtype:
        :enum:`EventsStreamType` or `unicode` or `None`
    """
    return request.param


@pytest.fixture()
def expected_stream_type(stream_type_param):
    """The stream type we expect to use.

    :rtype:
        `unicode`
    """
    if stream_type_param is None:
        return UserEventsStreamType.ALL
    return stream_type_param


@pytest.fixture()
def stream_type_kwargs(stream_type_param):
    """The kwargs for stream_type to pass when invoking a method on `Events`.

    :rtype:
        `dict`
    """
    if stream_type_param:
        return {'stream_type': stream_type_param}
    return {}


@pytest.fixture()
def expected_stream_type_params(expected_stream_type):
    """The stream_type-related params that we expect to pass to request methods.

    :rtype:
        :class:`OrderedDict`
    """
    return OrderedDict(stream_type=expected_stream_type)


@pytest.fixture()
def empty_events_response(final_stream_position):
    # pylint:disable=redefined-outer-name
    mock_box_response = Mock(BoxResponse)
    mock_network_response = Mock(DefaultNetworkResponse)
    mock_box_response.network_response = mock_network_response
    mock_box_response.json.return_value = mock_json = {'next_stream_position': final_stream_position, 'entries': []}
    mock_box_response.content = json.dumps(mock_json).encode()
    mock_box_response.status_code = 200
    mock_box_response.ok = True
    return mock_box_response


@pytest.fixture()
def long_poll_url(test_url, expected_stream_type_params):
    return urlunsplit(('', '', test_url, urlencode(expected_stream_type_params), ''))


@pytest.fixture()
def retry_timeout():
    return 610


@pytest.fixture()
def options_response_entry(long_poll_url, retry_timeout):
    return {'url': long_poll_url, 'retry_timeout': retry_timeout}


@pytest.fixture()
def options_response(options_response_entry, make_mock_box_request):
    # pylint:disable=redefined-outer-name
    mock_box_response, _ = make_mock_box_request(
        response={'entries': [options_response_entry]},
    )
    return mock_box_response


@pytest.fixture()
def new_change_long_poll_response(make_mock_box_request):
    mock_box_response, _ = make_mock_box_request(
        response={'message': 'new_change'},
    )
    return mock_box_response


@pytest.fixture()
def reconnect_long_poll_response(make_mock_box_request):
    mock_box_response, _ = make_mock_box_request(
        response={'message': 'reconnect'},
    )
    return mock_box_response


@pytest.fixture()
def max_retries_long_poll_response(make_mock_box_request):
    mock_box_response, _ = make_mock_box_request(
        response={'message': 'max_retries'},
    )
    return mock_box_response


@pytest.fixture()
def mock_event_json():
    return {
        "type": "event",
        "event_id": "f82c3ba03e41f7e8a7608363cc6c0390183c3f83",
        "source": {
            "type": "folder",
            "id": "11446498",
        },
    }


@pytest.fixture()
def events_response(initial_stream_position, mock_event_json, make_mock_box_request):
    # pylint:disable=redefined-outer-name
    mock_box_response, _ = make_mock_box_request(
        response={"next_stream_position": initial_stream_position, "entries": [mock_event_json]},
    )
    return mock_box_response


def test_get_events(
        test_events,
        mock_box_session,
        events_response,
        stream_type_kwargs,
        expected_stream_type_params,
):
    # pylint:disable=redefined-outer-name
    expected_url = test_events.get_url()
    mock_box_session.get.return_value = events_response
    events = test_events.get_events(**stream_type_kwargs)
    assert 'next_stream_position' in events
    mock_box_session.get.assert_any_call(
        expected_url,
        params=dict(limit=100, stream_position=0, **expected_stream_type_params),
    )
    event_entries = events['entries']
    assert event_entries == events_response.json.return_value['entries']
    for event in event_entries:
        assert isinstance(event, Event)


def test_get_long_poll_options(
        mock_box_session,
        test_events,
        stream_type_kwargs,
        expected_stream_type_params,
        options_response,
        options_response_entry,
):
    expected_url = test_events.get_url()
    mock_box_session.options.return_value = options_response
    long_poll_options = test_events.get_long_poll_options(**stream_type_kwargs)
    mock_box_session.options.assert_called_with(expected_url, params=expected_stream_type_params)
    assert long_poll_options == options_response_entry


def test_generate_events_with_long_polling(
        test_events,
        mock_box_session,
        events_response,
        empty_events_response,
        initial_stream_position,
        long_poll_url,
        retry_timeout,
        options_response,
        new_change_long_poll_response,
        reconnect_long_poll_response,
        max_retries_long_poll_response,
        mock_event_json,
        stream_type_kwargs,
        expected_stream_type,
        expected_stream_type_params,
):
    # pylint:disable=redefined-outer-name
    expected_url = test_events.get_url()
    mock_box_session.options.return_value = options_response
    mock_box_session.get.side_effect = [
        events_response,  # initial call to get now stream position
        Timeout,
        reconnect_long_poll_response,
        max_retries_long_poll_response,
        new_change_long_poll_response,
        events_response,
        new_change_long_poll_response,
        empty_events_response,
    ]
    events = test_events.generate_events_with_long_polling(**stream_type_kwargs)
    assert next(events) == Event(mock_event_json)
    with pytest.raises(StopIteration):
        next(events)
    events.close()
    mock_box_session.options.assert_called_with(expected_url, params=expected_stream_type_params)
    mock_box_session.get.assert_any_call(expected_url, params={'stream_position': 'now', 'limit': 0, 'stream_type': expected_stream_type})
    assert '/events' in expected_url
    mock_box_session.get.assert_any_call(
        expected_url,
        params=dict(limit=100, stream_position=initial_stream_position, **expected_stream_type_params),
    )
    mock_box_session.get.assert_any_call(
        long_poll_url,
        timeout=retry_timeout,
        params={'stream_position': initial_stream_position},
    )
